import logging
from graph_add_op import CHGraph
from clickhouse_driver import Client
from graph_op import CHGraph as ch
import Longger_design

# LOG_FORMAT = "%(asctime)s:%(levelname)s:%(name)s:%(funcName)s:%(message)s"
# logging.basicConfig(filename='./CHGraph.log', level=logging.INFO, format=LOG_FORMAT)
# logger = logging.getLogger('ModelService')
logger = Longger_design.get_logger()

operation_dict = {'avg': 'avg', 'sum': 'sum', 'min': 'min', 'max': 'max', 'count': 'count', }


def get_client(clickhouse_connect):
    graph_client = Client(host=clickhouse_connect["ip"])
    return graph_client


def disconnect_client(client):
    if client:
        client.disconnect()


class ModelService(object):
    # 切换图空间
    def use_graph(self, graph_name, db):
        res = db.use_tables(graph_name)
        print(res)
        if res is None:
            logger.warning("graph name [" + graph_name + "] does not exist")
            return
        else:
            self.graph_name = graph_name
            self.graph_cfg = res
            logger.info("use graph [" + graph_name + "] done")

    # 根据条件过滤查询子图
    def search_subgraph_by_condition(self, data,
                                     config_param):
        if "subGraph" in data.keys():
            graph_name = data["subGraph"]
        else:
            return None
        # if "fieldList" in data.keys():
        #     fieldList = data["fieldList"]
        #     fields = ",".join(fieldList)
        #
        edge_types = None
        if "edgeTypes" in data.keys():
            edge_types = data["edgeTypes"]

        node_types = None
        if "nodeTypes" in data.keys():
            node_types = data["nodeTypes"]

        db = config_param["db"]
        # graph_client = config_param["graphClient"]
        clickhouse_connect = config_param["clickhouse_connect"]
        graph_client = get_client(clickhouse_connect)
        graph = CHGraph(graph_client)
        self.use_graph(graph_name, db)
        ######
        edges = self.graph_cfg["edges"]
        vertexes = self.graph_cfg["vertexes"]

        if "edgeConditions" in data.keys():
            edge_conditions = data["edgeConditions"]
            edge_condition_dict, edge_order_dict, special_sql_dict = ConditionOperation().conditionalOperation(
                edge_conditions, edges)
        else:
            edge_condition_dict, edge_order_dict = {}, {}
        if "nodeConditions" in data.keys():
            node_conditions = data["nodeConditions"]
            node_condition_dict, node_order_dict, special_sql_dict = ConditionOperation().conditionalOperation(
                node_conditions, vertexes)
        else:
            node_condition_dict, node_order_dict = {}, {}

        if "fieldList" in data.keys():
            ext_field_list = data["fieldList"]
        else:
            ext_field_list = None
        if "resultType" in data.keys():
            result_type = data["resultType"]
        else:
            result_type = None
        data = {}
        path_data = {}
        edge_data_list = []
        vertexes_data_list = []

        if edge_types:
            edge_data_list = executeController(edge_types, edges, edge_condition_dict, edge_order_dict, graph, "edge",
                                               ext_field_list, result_type)
        else:
            # edge_data_list = executeController(edges, edges, edge_condition_dict, edge_order_dict, graph, "edge")
            pass
        if node_types:
            vertexes_data_list = executeController(node_types, vertexes, node_condition_dict, node_order_dict, graph,
                                                   "vertexes", ext_field_list, result_type)
        else:
            # vertexes_data_list = executeController(vertexes, vertexes, node_condition_dict, node_order_dict, graph,
            #                                      "vertexes")
            pass

        path_data["graphEdges"] = edge_data_list
        path_data["graphNodes"] = vertexes_data_list
        data["pathList"] = path_data
        # 释放链接
        disconnect_client(graph_client)
        return data

    # 时间轴获取点边的模型
    def time_line_search(self, data, config_param):
        res_data = {}
        if "subGraph" in data.keys():
            graph_name = data["subGraph"]
        else:
            return None
        # if "fieldList" in data.keys():
        #     fieldList = data["fieldList"]
        #     fields = ",".join(fieldList)
        #
        edge_types = None
        if "edgeTypes" in data.keys():
            edge_types = data["edgeTypes"]

        node_types = None
        if "nodeTypes" in data.keys():
            node_types = data["nodeTypes"]

        db = config_param["db"]
        # graph_client = config_param["graphClient"]
        clickhouse_connect = config_param["clickhouse_connect"]
        graph_client = get_client(clickhouse_connect)
        graph = CHGraph(graph_client)
        self.use_graph(graph_name, db)
        ######
        edges = self.graph_cfg["edges"]
        vertexes = self.graph_cfg["vertexes"]
        if "edgeConditions" in data.keys() or "nodeConditions" in data.keys():
            if "edgeConditions" in data.keys():
                edge_conditions = data["edgeConditions"]
                edge_condition_dict, edge_order_dict, special_sql_dict = ConditionOperation().conditionalOperation(
                    edge_conditions, edges)
                if "sql" in special_sql_dict:
                    res = execute_graph_sql(graph, special_sql_dict["sql"])
            if "nodeConditions" in data.keys():
                node_conditions = data["nodeConditions"]
                node_condition_dict, node_order_dict, special_sql_dict = ConditionOperation().conditionalOperation(
                    node_conditions, vertexes)
                if "sql" in special_sql_dict:
                    res = execute_graph_sql(graph, special_sql_dict["sql"])
        if res:
            res_dict = res["data"]
            res_data["columns"] = res_dict["schema"]
            res_data["rowList"] = res_dict["detail"]
        # 释放链接
        disconnect_client(graph_client)
        return res_data

    # 统计任意两点间的边
    def count_src_dst_round(self, data, config_param):
        if "subGraph" in data.keys():
            graph_name = data["subGraph"]
        else:
            return None
        # if "fieldList" in data.keys():
        #     fieldList = data["fieldList"]
        #     fields = ",".join(fieldList)
        #
        edge_types = None
        if "edgeTypes" in data.keys():
            edge_types = data["edgeTypes"]

        db = config_param["db"]
        # graph_client = config_param["graphClient"]
        clickhouse_connect = config_param["clickhouse_connect"]
        graph_client = get_client(clickhouse_connect)
        graph = CHGraph(graph_client)
        self.use_graph(graph_name, db)
        ######
        edges = self.graph_cfg["edges"]

        if "edgeConditions" in data.keys():
            edge_conditions = data["edgeConditions"]
            edge_condition_dict, edge_order_dict, special_sql_dict = ConditionOperation().conditionalOperation(
                edge_conditions, edges)
        else:
            edge_condition_dict, edge_order_dict = {}, {}

        if edge_types:
            edge_data_list = executeController(edge_types, edges, edge_condition_dict, edge_order_dict, graph,
                                               "edgeCount")
        else:
            edge_data_list = executeController(edges.keys(), edges, edge_condition_dict, edge_order_dict, graph,
                                               "edgeCount")
        res_data = {}
        res_list = []
        if edge_data_list:
            for res_dict in edge_data_list:
                res = res_dict["data"]
                res_data["columns"] = res["schema"]
                res_data["rowList"] = res["detail"]
                res_data["type"] = res_dict["type"]
                res_list.append(res_data)
        # 释放链接
        disconnect_client(graph_client)
        return res_list

    # 根据条件过滤查询子图
    def query_subgraph(self, data, config_params):

        if "subGraph" in data.keys():
            graphName = data["subGraph"]
            print(graphName)
        else:
            return None
        # graph = config_params["graph"]
        db = config_params["db"]
        # graph.use_graph(graphName, db)
        # graph_client = config_param["graphClient"]
        clickhouse_connect = config_params["clickhouse_connect"]
        graph_client = get_client(clickhouse_connect)
        graph = ch(graph_client)
        graph.use_graph(graphName, db)
        graph_cfg = db.use_tables(graphName)

        graphNodes = []
        for vertex in graph_cfg["vertexes"]:
            graphNode = {}
            dict1 = graph_cfg["vertexes"][vertex]
            tem = []
            tem.append(dict1["id"])
            tem.append(dict1["label"])
            for field in dict1["fields"]:
                tem.append(field)
                try:
                    vertex_query = graph.query_vertexes(
                        vertex,
                        [""],
                        tem,
                        "df"
                    )
                except Exception as e:
                    print(e)
                    return "vertex query failed"
                graphNode["type"] = vertex
                graphNode["data"] = vertex_query
                graphNodes.append(graphNode)
        graphEdges = []
        for edge in graph_cfg["edges"]:
            graphEdge = {}
            dict2 = graph_cfg["edges"][edge]
            tem = []
            tem.append(dict2["src"])
            tem.append(dict2["dst"])
            tem.append(dict2["rank"])
            for field in dict2["fields"]:
                tem.append(field)
            try:
                edges_result = graph.query_edges(
                    edge,
                    [""],
                    tem,
                    "df"
                )
            except Exception as e:
                print(e)
                return "edge query failed"
        graphEdge["type"] = edge
        graphEdge["data"] = edges_result
        graphEdge["id"] = edge

        graphEdges.append(graphEdge)
        res_list = {}
        res_list["pathList"] = {}
        res_list["pathList"]["graphEdges"] = graphEdges
        res_list["pathList"]["graphNodes"] = graphNodes
        return res_list

    # 统计子图中点或者边的数量
    def statistics_operation_function(self, data, config_param):
        data = dispose_parse_params(data)
        if "subGraph" in data.keys():
            graph_name = data["subGraph"]
        else:
            return None
        edge_types = None
        if "type" in data.keys():
            graph_type = data["type"]
        else:
            graph_type = None
        if "orderTypes" in data.keys():
            orderTypes = data["orderTypes"]
        else:
            orderTypes = ""
        db = config_param["db"]
        clickhouse_connect = config_param["clickhouse_connect"]
        graph_client = get_client(clickhouse_connect)
        graph = CHGraph(graph_client)
        ###处理参数
        operations = {"1": "count"}
        group_field = []
        order_operation = orderTypes
        field_list = []
        self.use_graph(graph_name, db)
        if graph_type:
            schemas = self.graph_cfg[graph_type]
            res = None
            for schema in schemas:
                execute_sql = joinSql(operations, group_field, order_operation,
                                      schema, schemas, field_list, graph_type)
                result_dict = execute_graph_sql(graph, execute_sql)
                res = self.dispose_statistics_operation_res(result_dict, res, schema)
            result = {graph_type: res}

        else:
            edges = self.graph_cfg["edges"]
            vertexes = self.graph_cfg["vertexes"]
            res_edge = None
            res_vertex = None
            for edge in edges:
                execute_sql = joinSql(operations, group_field, order_operation,
                                      edge, edges, field_list, "edges")
                result_dict = execute_graph_sql(graph, execute_sql)
                res_edge = self.dispose_statistics_operation_res(result_dict, res_edge, edge)
            for vertex in vertexes:
                execute_sql = joinSql(operations, group_field, order_operation,
                                      vertex, vertexes, field_list, "vertexes")
                result_dict = execute_graph_sql(graph, execute_sql)
                res_vertex = self.dispose_statistics_operation_res(result_dict, res_vertex, vertex)
            result = {"vertexes": res_vertex, "edges": res_edge}
        return result

    def dispose_statistics_operation_res(self, result_dict, res, type):
        if "typeValue" not in result_dict["data"]["schema"]:
            result_dict["data"]["schema"].append("typeValue")
        result_dict["data"]["detail"][0].append(type)
        if res:
            res["data"].result_dict["data"][0]
        else:
            res = result_dict
        return res

    # 统计子图中点或者边的多个属性
    def statistics_operation_attributes_function(self, data, config_param):
        data = dispose_parse_params(data)
        if "subGraph" in data.keys():
            graph_name = data["subGraph"]
        else:
            return None
        if "type" in data.keys():
            type_value = data["type"]
        else:
            type_value = None
        if "attributes" in data.keys():
            attributes = data["attributes"]
        else:
            return None
        if "orderTypes" in data.keys():
            orderTypes = data["orderTypes"]
        else:
            orderTypes = ""

        # 获取客户端
        db = config_param["db"]
        clickhouse_connect = config_param["clickhouse_connect"]
        graph_client = get_client(clickhouse_connect)
        graph = CHGraph(graph_client)

        # 更新子图
        self.use_graph(graph_name, db)
        edges = self.graph_cfg["edges"]
        vertexes = self.graph_cfg["vertexes"]

        # 处理参数
        attribute_list = attributes.split(",")
        if orderTypes:
            order_list = orderTypes.split(",")
        else:
            order_list = ""
        res_list = []
        for i in range(len(attribute_list)):

            if order_list:
                order_operation = order_list[i]
            else:
                order_operation = ""
            field_list = []
            data_param = {"subGraph": graph_name, "type": type_value, "attribute": attribute_list[i],
                          "orderTypes": order_operation}
            if type_value in edges:
                index = edges[type_value]["fields"].index(attribute_list[i])
                field_type = edges[type_value]["types"][index]
                if field_type == "String":
                    data_param["statistics"] = "count"
                elif field_type == "Date":
                    data_param["statistics"] = "count"

            if type_value in vertexes:
                index = vertexes[type_value]["fields"].index(attribute_list[i])
                field_type = vertexes[type_value]["types"][index]
                if field_type == "String":
                    data_param["statistics"] = "count"
                elif field_type == "Date":
                    data_param["statistics"] = "count"
            res_one = self.statistics_operation_attribute_function(data_param, config_param, graph=graph)
            res_list.append(res_one)

        return res_list

    # 统计点或者边的单个属性
    def statistics_operation_attribute_function(self, data, config_param, label_statistic=None, graph=None):
        data = dispose_parse_params(data)
        if "subGraph" in data.keys():
            graph_name = data["subGraph"]
        else:
            return None
        if "type" in data.keys():
            type_value = data["type"]
        else:
            type_value = None
        attribute = None
        if "attribute" in data.keys():
            attribute = data["attribute"]
        else:
            return None
        if "statistics" in data.keys():
            statistics = data["statistics"]
        else:
            statistics = "def"
        if "orderTypes" in data.keys():
            orderTypes = data["orderTypes"]
        else:
            orderTypes = ''
        # 处理参数
        attribute_arr = attribute.split(",")
        statistics_arr = statistics.split(",")
        if len(statistics_arr) != len(attribute_arr):
            return None
        operations = {}
        for i in range(len(statistics_arr)):
            operations[attribute_arr[i]] = statistics_arr[i]
        if "def" in statistics_arr:
            group_field = None
        else:
            group_field = attribute_arr
        order_operation = orderTypes
        field_list = []
        # 获取客户端
        db = config_param["db"]
        clickhouse_connect = config_param["clickhouse_connect"]
        graph_client = get_client(clickhouse_connect)
        if graph:
            pass
        else:
            graph = CHGraph(graph_client)
        # 更换子图
        self.use_graph(graph_name, db)
        edges = self.graph_cfg["edges"]
        vertexes = self.graph_cfg["vertexes"]
        if type_value in edges:
            edge_schema = edges[type_value]
            if label_statistic:
                group_field = [edge_schema["src"], edge_schema["dst"]]
            execute_sql = joinSql(operations, group_field, order_operation,
                                  type_value, edges, field_list, "edges")
        elif type_value in vertexes:
            vertex_schema = vertexes[type_value]
            if label_statistic:
                group_field = [vertex_schema["id"]]
            execute_sql = joinSql(operations, group_field, order_operation,
                                  type_value, vertexes, field_list, "vertexes")
        else:
            return None
        result_dict = execute_graph_sql(graph, execute_sql)

        return result_dict


# 拼接sql,查询表数量
def joinSql(operations, group_field, order_operation,
            type_vertex, schema, field_list, subgraph_type):
    if field_list:
        for operation in operations:
            if operation in field_list:
                pass
            else:
                field_list.append(operation)
    else:
        for operation in operations:
            field_list.append(operation)

    if field_list:
        if group_field:
            for field in group_field:
                if field in field_list:
                    pass
                else:
                    field_list.append(field)
    else:
        for field in group_field:
            field_list.append(field)
    operList = statisticsOperation(operations)
    operStr = ','.join(operList)
    group_s = ""
    operStr_new = operStr
    if group_field:
        group_str = ','.join(group_field)
        operStr_new = group_str + "," + operStr_new
        group_s = " group by " + group_str
    main_sql = mainSqlValue(type_vertex, schema, field_list, subgraph_type)
    if main_sql:
        sql = "select " + operStr_new + " from (" + main_sql + ")" + group_s + " order by " + operStr + "  " + order_operation
        return sql
    else:
        return


# 处理Statistics函数
def statisticsOperation(operations):
    operList = []
    for operation in operations:
        oper = operations[operation]
        if oper in operation_dict:
            operList.append(operation_dict[oper] + '(' + operation + ')')
        else:
            operList.append(operation)
    return operList


# 点子图主sql
def mainSqlValue(type_value, schema, field_list, subgraph_type):
    if type_value in schema:
        value = schema[type_value]
        if subgraph_type == "vertexes":
            if field_list:
                fields = ",".join(field_list)
                sql = "select distinct " + value["id"] + "," + fields + " from " + value["db"] + "." + value["table"]
            else:
                sql = "select distinct " + value["id"] + "," + " from " + value["db"] + "." + value["table"]
            return sql
        else:
            if field_list:
                fields = ",".join(field_list)
                sql = "select distinct " + value["src"] + "," + value["dst"] + "," + value["rank"] + "," + fields \
                      + " from " + value["db"] + "." + value["table"]
            else:
                sql = "select distinct " + value["src"] + "," + value["dst"] + "," + value["rank"] + "," + " from " \
                      + value["db"] + "." + value["table"]
            return sql

    else:
        return


# 执行多条sql，并拼接返回
def executeController(array, schema, condition_dict, order_dict, graph, type,
                      ext_field_list=None, result_type=None):
    result = []
    for key in array:
        data = schema[key]
        condition = condition_splice(key, condition_dict)
        order = condition_splice(key, order_dict)
        if type == "edge":
            main_sql = edges_splice(data, ext_field_list)
        elif type == "edgeCount":
            main_sql = edges_count_splice(data)
            group_splice = edges_group_splice(data)
        else:
            main_sql = vertexes_splice(data, ext_field_list)
        if main_sql:
            # sql = main_sql + condition + order + " limit 10"
            if type == "edgeCount":
                sql = main_sql + condition + group_splice + order
            else:
                sql = main_sql + condition + order
            vertexes_data = execute_graph_sql(graph, sql, key, result_type)
            result.append(vertexes_data)
    return result


def edges_group_splice(edges_schema):
    if "src" in edges_schema:
        src = edges_schema["src"]
    else:
        return

    if "dst" in edges_schema:
        dst = edges_schema["dst"]
    else:
        return
    group_splice = " group by " + src + "," + dst + " "
    return group_splice


# 组合sql执行，返回图对象
def execute_graph_sql(graph, sql, type=None, result_type=None):
    result_dict = {}
    data = {}
    logger.info("sql:" + sql)
    res = graph.execute(sql)
    if result_type:
        result_dict["data"] = res
    else:
        schemas = res.columns.values.tolist()
        field_data = res.values.tolist()
        data["schema"] = schemas
        data["detail"] = field_data
        result_dict["data"] = data
    if type:
        result_dict["type"] = type
    return result_dict


# 拼接条件
def condition_splice(type, condition={}):
    if len(condition) > 0 and type in condition:
        return condition[type]
    return ""


# 拼接点主体查询sql
def vertexes_splice(vertexes_schema, ext_field_list=None):
    if "db" in vertexes_schema:
        db = vertexes_schema["db"]
    else:
        db = "default"

    if "table" in vertexes_schema:
        table = vertexes_schema["table"]
    else:
        return

    if "id" in vertexes_schema:
        id = vertexes_schema["id"]
    else:
        return

    if "label" in vertexes_schema:
        label = vertexes_schema["label"]
    else:
        # label = ""
        return
    ext_field = None
    if ext_field_list:
        pass
    else:
        ext_field_list = vertexes_schema["fields"]
    if id in ext_field_list:
        ext_field_list.remove(id)
    if label in ext_field_list:
        ext_field_list.remove(label)
    if ext_field_list:
        ext_field = ",".join(ext_field_list)
    if ext_field:
        main_sql = "select distinct " + id + "," + label + ", " + ext_field + " from " + db + "." + table
    else:
        main_sql = "select distinct " + id + "," + label + " from " + db + "." + table
    # main_sql = "select " + id + " from " + db + "." + table
    return main_sql


# 拼接边主体查询sql
def edges_splice(edges_schema, ext_field_list=None):
    if "db" in edges_schema:
        db = edges_schema["db"]
    else:
        db = "default"

    if "table" in edges_schema:
        table = edges_schema["table"]
    else:
        return

    if "src" in edges_schema:
        src = edges_schema["src"]
    else:
        return

    if "dst" in edges_schema:
        dst = edges_schema["dst"]
    else:
        return
    if "rank" in edges_schema:
        rank = edges_schema["rank"]
    else:
        return
    ext_field = None
    if ext_field_list:
        pass
    else:
        ext_field_list = edges_schema["fields"]
    if src in ext_field_list:
        ext_field_list.remove(src)
    if dst in ext_field_list:
        ext_field_list.remove(dst)
    if rank in ext_field_list:
        ext_field_list.remove(rank)
    if ext_field_list:
        ext_field = ",".join(ext_field_list)
    if ext_field:
        main_sql = "select distinct " + src + ", " + dst + "," + rank + " ," + ext_field + "  from  " + db + "." + table
    else:
        main_sql = "select distinct " + src + ", " + dst + "," + rank + "  from  " + db + "." + table
    return main_sql


# 拼接边主体查询sql
def edges_count_splice(edges_schema):
    if "db" in edges_schema:
        db = edges_schema["db"]
    else:
        db = "default"

    if "table" in edges_schema:
        table = edges_schema["table"]
    else:
        return

    if "src" in edges_schema:
        src = edges_schema["src"]
    else:
        return

    if "dst" in edges_schema:
        dst = edges_schema["dst"]
    else:
        return
    if "rank" in edges_schema:
        rank = edges_schema["rank"]
    else:
        return
    main_sql = "select " + src + ", " + dst + ", count(1) count  from  " + db + "." + table
    return main_sql


# 拼接order by
def orderAttributeOperation(orderAttribute):
    if orderAttribute:
        order = ",".join(orderAttribute)
        return "order by " + order


# 拼接group by
# 暂时不用，没有处理聚合函数
def groupAttributeOperation(groupAttribute):
    if groupAttribute:
        group = ",".join(groupAttribute)
        return "group by " + group


# 遍历字符串并加上单引号
def foreachStringArray2String(array=[]):
    if array:
        str = None
        for tmp in array:
            if str:
                str = str + "'" + tmp + "',"
            else:
                str = "'" + tmp + "',"
        return str[0:len(str)]
    return


# 处理parse.parse_qs方法获取的get参数
def dispose_parse_params(data):
    for key in data.keys():
        if isinstance(data[key], list):
            data[key] = data[key][0]
    return data


class ConditionOperation(object):
    singular = ["=", ">", "<", "like"]
    allCondition = ["in"]
    twoCondition = ["between"]
    special = ["timeType"]
    attribute_type_num = "num"
    attribute_type_time = "time"

    # 合并conditional条件
    def conditionalOperation(self, typeConditions, schemas={}):

        typeConditionDict = {}
        typeOrderDict = {}
        typeSpecialSql = {}
        if typeConditions:
            for typeCondition in typeConditions:
                val = None
                type = typeCondition["type"]
                schema = schemas[type]

                if "srcNodes" in typeCondition:
                    srcNodes = typeCondition["srcNodes"]
                    if srcNodes:
                        if "src" in schema:
                            if schema["src_data_type"] == self.attribute_type_num:
                                src_node = ",".join(srcNodes)
                            else:
                                src_node = foreachStringArray2String(srcNodes)
                            if val:
                                val = val + schema["src"] + " in(" + src_node + ") and "
                            else:
                                val = " where " + schema["src"] + " in(" + src_node + ") and "

                if "destNodes" in typeCondition:
                    destNodes = typeCondition["destNodes"]
                    if destNodes:
                        if "dst" in schema:
                            if schema["dst_data_type"] == self.attribute_type_num:
                                dest_node = ",".join(destNodes)
                            else:
                                dest_node = foreachStringArray2String(destNodes)
                            if val:
                                val = val + schema["dst"] + " in(" + dest_node + ") and "
                            else:
                                val = " where " + schema["dst"] + " in(" + dest_node + ") and "
                if "id" in typeCondition:
                    srcNodes = typeCondition["id"]
                    if srcNodes:
                        if "id" in schema:
                            if schema["id_data_type"] == self.attribute_type_num:
                                ids = ",".join(srcNodes)
                            else:
                                ids = foreachStringArray2String(srcNodes)
                            if val:
                                val = val + schema["id"] + " in(" + ids + ") and "
                            else:
                                val = " where " + schema["id"] + " in(" + ids + ") and "
                if "conditions" in typeCondition:
                    conditions = typeCondition["conditions"]
                    val_sql = val
                    val_dict = self.splice_condition(conditions, schema, val_sql)
                    if "sql" in val_dict:
                        typeSpecialSql["sql"] = val_dict["sql"]
                    val = val_dict["val"]

                if val:
                    typeConditionDict[type] = val + " 1=1"

                if "orderAttribute" in typeCondition:
                    type_order = typeCondition["orderAttribute"]
                    order_val = orderAttributeOperation(type_order)
                    typeOrderDict[type] = order_val
            return typeConditionDict, typeOrderDict, typeSpecialSql

        return

    # 拼接条件
    def splice_condition(self, conditions, schema, val_sql=None):
        if conditions:
            tem_dict = {}
            if val_sql:
                val = val_sql
            else:
                val = " where "
            for condition in conditions:
                symbol = condition["symbol"]
                attribute_type = condition["type"]
                if symbol in self.singular:
                    if attribute_type == self.attribute_type_num:
                        val = val + condition["attribute"] + condition["symbol"] \
                              + condition["conditional"][0] + " and "
                    elif attribute_type == self.attribute_type_time:
                        val = val + " formatDateTime(" + condition["attribute"] + ",'%F %T') " + condition["symbol"] \
                              + " '" + condition["conditional"][0] + "' and "
                    else:
                        val = val + condition["attribute"] + condition["symbol"] \
                              + "'" + condition["conditional"][0] + "' and "

                if symbol in self.allCondition:
                    if attribute_type == self.attribute_type_num:
                        val = val + condition["attribute"] + condition["symbol"] + "(" \
                              + ",".join(condition["conditional"]) + ")" + " and "
                    else:
                        val = val + condition["attribute"] + condition["symbol"] + "(" \
                              + foreachStringArray2String(condition["conditional"]) + ")" + " and "
                if symbol in self.twoCondition:
                    if attribute_type == self.attribute_type_num:
                        val = val + condition["attribute"] + " between " \
                              + condition["conditional"][0] + " and " \
                              + condition["conditional"][1] + " and "
                    elif attribute_type == self.attribute_type_time:
                        val = val + " formatDateTime(" + condition["attribute"] + ",'%F %T') " + condition["symbol"] \
                              + " '" + condition["conditional"][0] + "' and "
                    else:
                        val = val + condition["attribute"] + " between '" \
                              + condition["conditional"][0] + "' and '" \
                              + condition["conditional"][1] + "' and "
                if symbol in self.special:
                    sql = self.time_line_count_sql(condition, schema, val_sql)
                    tem_dict["sql"] = sql
            tem_dict["val"] = val
            return tem_dict

    # 处理特殊sql的拼接
    def time_line_count_sql(self, condition, schema, val_sql):
        if "id" in schema and "label" in schema:
            attribute = schema["id"] + " , " + schema["label"]
        else:
            attribute = schema["src"] + " , " + schema["dst"] + " , " + schema["rank"]
        if val_sql:
            val_sql = val_sql + " 1=1 "
        else:
            val_sql = " where 1=1 "
        if condition["conditional"][0] == "year":
            sql = "select toYear(" + condition["attribute"] + ") year,count(1) count from ( select distinct " \
                  + attribute + "," + condition["attribute"] + " from " + schema["db"] + "." + schema[
                      "table"] + val_sql + " ) group by year order by year "
        elif condition["conditional"][0] == "quarter":
            sql = "select concat(toString(year),\' \',toString(quarter)) yearQuarter ,count from (select toYear(" + \
                  condition["attribute"] + ") year,toQuarter(" + condition["attribute"] \
                  + ") quarter ,count(1) count from   ( select distinct  " + attribute + "," + condition[
                      "attribute"] + " from " \
                  + schema["db"] + "." + schema["table"] \
                  + val_sql \
                  + " )  group by year,quarter order by year,quarter) "
        elif condition["conditional"][0] == "month":
            sql = "select concat(toString(year),\' \',toString(month)) yearMonth, count from ( select toYear(" + \
                  condition["attribute"] + ") year,toMonth(" + condition["attribute"] \
                  + ") month ,count(1) count from  ( select distinct " + attribute + "," + condition[
                      "attribute"] + " from " \
                  + schema["db"] + "." + schema["table"] \
                  + val_sql \
                  + " )group by year,month order by year,month)"
        elif condition["conditional"][0] == "week":
            sql = "select concat(toString(yw),\' \',toString(week)) ywWeek, count from (select toYearWeek(" + \
                  condition[
                      "attribute"] + ") yw, toDayOfWeek(" + condition["attribute"] + \
                  ") week ,count(1) count from   ( select distinct  " + attribute + "," + condition[
                      "attribute"] + " from " \
                  + schema["db"] + "." + schema["table"] \
                  + val_sql \
                  + ") group by yw,week order by yw,week )"
        elif condition["conditional"][0] == "day":
            sql = "select formatDateTime(" + condition[
                "attribute"] + ",'%F') day ,count(1) count from  ( select distinct  " \
                  + attribute + "," + condition["attribute"] + " from " \
                  + schema["db"] + "." + schema["table"] + val_sql + " ) group by day order by day"
        elif condition["conditional"][0] == "hours":
            sql = "select formatDateTime(" + condition[
                "attribute"] + ",'%F %H') h ,count(1) count from ( select distinct  " \
                  + attribute + "," + condition["attribute"] + " from " \
                  + schema["db"] + "." + schema["table"] + val_sql + " )group by h order by h"
        elif condition["conditional"][0] == "minute":
            sql = "select formatDateTime(" + condition[
                "attribute"] + ",'%F %R') m ,count(1) count from  ( select distinct  " \
                  + attribute + "," + condition["attribute"] + " from " \
                  + schema["db"] + "." + schema["table"] + val_sql + " )group by m order by m"
        else:
            sql = "select formatDateTime(" + condition[
                "attribute"] + ",'%F') day ,count(1) count from  ( select distinct  " \
                  + attribute + "," + condition["attribute"] + " from " \
                  + schema["db"] + "." + schema["table"] + val_sql + " )group by day order by day"

        #     if condition["conditional"][0] == "year":
        #         sql = "select toYear(" + condition["attribute"] + ") year,count(1) count from ( select distinct " \
        #              + attribute + "," + condition["attribute"] + " from " + schema["db"] + "." + schema["table"] + val_sql + " ) group by year order by year "
        #     elif condition["conditional"][0] == "quarter":
        #         sql = "select concat(toString(year),\' \',toString(quarter)) yearQuarter ,count from (select toYear(" + \
        #               condition["attribute"] + ") year,toQuarter(" + condition["attribute"] \
        #               + ") quarter ,count(1) count from   ( select distinct  " + attribute + "," + condition["attribute"] + " from " \
        #               + schema["db"] + "." + schema["table"] \
        #               + val_sql \
        #               + " )  group by year,quarter order by year,quarter) "
        #     elif condition["conditional"][0] == "month":
        #         sql = "select concat(toString(year),\' \',toString(month)) yearMonth, count from ( select toYear(" + \
        #               condition["attribute"] + ") year,toMonth(" + condition["attribute"] \
        #               + ") month ,count(1) count from  ( select distinct " + attribute + "," + condition["attribute"] + " from "\
        #               + schema["db"] + "." + schema["table"] \
        #               + val_sql \
        #               + " )group by year,month order by year,month)"
        #     elif condition["conditional"][0] == "week":
        #         sql = "select concat(toString(yw),\' \',toString(week)) ywWeek, count from (select toYearWeek(" + \
        #               condition[
        #                   "attribute"] + ") yw, toDayOfWeek(" + condition["attribute"] + \
        #               ") week ,count(1) count from   ( select distinct  " + attribute + "," + condition["attribute"] + " from "\
        #               + schema["db"] + "." + schema["table"] \
        #               + val_sql \
        #               + ") group by yw,week order by yw,week )"
        #     elif condition["conditional"][0] == "day":
        #         sql = "select formatDateTime(" + condition["attribute"] + ",'%F') day ,count(1) count from  ( select distinct  " \
        #               + attribute + "," + condition["attribute"] + " from " \
        #               + schema["db"] + "." + schema["table"] + val_sql + ") group by day order by day"
        #     elif condition["conditional"][0] == "hours":
        #         sql = "select formatDateTime(" + condition["attribute"] + ",'%F %H') h ,count(1) count from ( select distinct  " \
        #               + schema["db"] + "." + schema["table"] + val_sql + " group by h order by h"
        #     elif condition["conditional"][0] == "minute":
        #         sql = "select formatDateTime(" + condition["attribute"] + ",'%F %R') m ,count(1) count from  ( select distinct  " \
        #               + attribute + "," + condition["attribute"] + " from " \
        #               + schema["db"] + "." + schema["table"] + val_sql + " )group by m order by m"
        #     else:
        #         sql = "select formatDateTime(" + condition["attribute"] + ",'%F') day ,count(1) count from  ( select distinct  " \
        #               + attribute + "," + condition["attribute"] + " from " \
        #               + schema["db"] + "." + schema["table"] + val_sql + " )group by day order by day"
        # else:
        #     if condition["conditional"][0] == "year":
        #         sql = "select toYear(" + condition["attribute"] + ") year,count(1) count from ( select distinct " \
        #              + attribute + "," + condition["attribute"] + " from " + schema["db"] + "." + schema["table"] + " ) group by year order by year "
        #     elif condition["conditional"][0] == "quarter":
        #         sql = "select concat(toString(year),\' \',toString(quarter)) yearQuarter ,count from (select toYear(" + \
        #               condition["attribute"] + ") year,toQuarter(" + condition["attribute"] \
        #               + ") quarter ,count(1) count from   ( select distinct  " + attribute + "," + condition["attribute"] + " from " \
        #               + schema["db"] + "." + schema["table"] \
        #               + " )  group by year,quarter order by year,quarter) "
        #     elif condition["conditional"][0] == "month":
        #         sql = "select concat(toString(year),\' \',toString(month)) yearMonth, count from ( select toYear(" + \
        #               condition["attribute"] + ") year,toMonth(" + condition["attribute"] \
        #               + ") month ,count(1) count from  ( select distinct " + attribute + "," + condition["attribute"] + " from "\
        #               + schema["db"] + "." + schema["table"] \
        #               + " )group by year,month order by year,month)"
        #     elif condition["conditional"][0] == "week":
        #         sql = "select concat(toString(yw),\' \',toString(week)) ywWeek, count from (select toYearWeek(" + \
        #               condition[
        #                   "attribute"] + ") yw, toDayOfWeek(" + condition["attribute"] + \
        #               ") week ,count(1) count from   ( select distinct  " + attribute + "," + condition["attribute"] + " from "\
        #               + schema["db"] + "." + schema["table"] \
        #               + ") group by yw,week order by yw,week )"
        #     elif condition["conditional"][0] == "day":
        #         sql = "select formatDateTime(" + condition["attribute"] + ",'%F') day ,count(1) count from  ( select distinct  " \
        #               + attribute + "," + condition["attribute"] + " from " \
        #               + schema["db"] + "." + schema["table"] + ") group by day order by day"
        #     elif condition["conditional"][0] == "hours":
        #         sql = "select formatDateTime(" + condition["attribute"] + ",'%F %H') h ,count(1) count from ( select distinct  " \
        #               + attribute + "," + condition["attribute"] + " from " \
        #               + schema["db"] + "." + schema["table"] + " )group by h order by h"
        #     elif condition["conditional"][0] == "minute":
        #         sql = "select formatDateTime(" + condition["attribute"] + ",'%F %R') m ,count(1) count from  ( select distinct  " \
        #               + attribute + "," + condition["attribute"] + " from " \
        #               + schema["db"] + "." + schema["table"] + " )group by m order by m"
        #     else:
        #         sql = "select formatDateTime(" + condition["attribute"] + ",'%F') day ,count(1) count from  ( select distinct  " \
        #               + attribute + "," + condition["attribute"] + " from " \
        #               + schema["db"] + "." + schema["table"] + " )group by day order by day"
        # if condition["conditional"][0] == "year":
        #     sql = "select toYear(" + condition["attribute"] + ") year,count(1) count from " \
        #           + schema["db"] + "." + schema["table"] + " group by year order by year "
        # elif condition["conditional"][0] == "quarter":
        #     sql = "select concat(toString(year),\' \',toString(quarter)) yearQuarter ,count from (select toYear(" + \
        #           condition["attribute"] + ") year,toQuarter(" + condition["attribute"] \
        #           + ") quarter ,count(1) count from " + schema["db"] + "." + schema["table"] \
        #           + " group by year,quarter order by year,quarter) "
        # elif condition["conditional"][0] == "month":
        #     sql = "select concat(toString(year),\' \',toString(month)) yearMonth, count from ( select toYear(" + \
        #           condition["attribute"] + ") year,toMonth(" + condition["attribute"] \
        #           + ") month ,count(1) count from " + schema["db"] + "." + schema["table"] \
        #           + " group by year,month order by year,month)"
        # elif condition["conditional"][0] == "week":
        #     sql = "select concat(toString(yw),\' \',toString(week)) ywWeek, count from (select toYearWeek(" + \
        #           condition[
        #               "attribute"] + ") yw, toDayOfWeek(" + condition["attribute"] + \
        #           ") week ,count(1) count from " + schema["db"] + "." + schema["table"] \
        #           + " group by yw,week order by yw,week )"
        # elif condition["conditional"][0] == "day":
        #     sql = "select formatDateTime(" + condition["attribute"] + ",'%F') day ,count(1) count from " \
        #           + schema["db"] + "." + schema["table"] + " group by day order by day"
        # elif condition["conditional"][0] == "hours":
        #     sql = "select formatDateTime(" + condition["attribute"] + ",'%F %H') h ,count(1) count from " \
        #           + schema["db"] + "." + schema["table"] + " group by h order by h"
        # elif condition["conditional"][0] == "minute":
        #     sql = "select formatDateTime(" + condition["attribute"] + ",'%F %R') m ,count(1) count from " \
        #           + schema["db"] + "." + schema["table"] + " group by m order by m"
        # else:
        #     sql = "select formatDateTime(" + condition["attribute"] + ",'%F') day ,count(1) count from " \
        #           + schema["db"] + "." + schema["table"] + " group by day order by day"
        return sql


def main():
    from clickhouse_driver import Client
    graphClient = Client(host="10.202.255.93", port="9090", user="default", password="root")
    from graph_op import CHGraph
    graph = CHGraph(graphClient)
    res = graph.execute("select * from anti_money_launder.transactions limit 10")
    list = res.columns.values.tolist();
    print(res.values.tolist())
    print(list)


def test():
    import data_load_config as config
    config_params = config.load_config()
    str(config_params["db"])
    print(str(config_params["db"]))
    dict_data = {"subGraph": "anti_money_launder"}
    model_service = ModelService()
    result_dict = model_service.search_subgraph_by_condition(dict_data, config_params)
    print(result_dict)


def test_function():
    import data_load_config as config
    config_params = config.load_config()
    str(config_params["db"])
    print(str(config_params["db"]))
    # dict_data = {"subGraph": "0b58bfda_7bf5_4661_ae21_5079057afe49", "type": "xiqianvertex",
    #              "attribute": "initial_deposit", "statistics": ""}
    # model_service = ModelService()
    # result_dict = model_service.statistics_operation_attribute_function(dict_data, config_params)
    # print(result_dict)
    # dict_data = {"subGraph": "0b58bfda_7bf5_4661_ae21_5079057afe49"}
    # model_service = ModelService()
    # res = model_service.statistics_operation_function(dict_data, config_params)
    # print(res)
    dict_data = {"subGraph": "0b58bfda_7bf5_4661_ae21_5079057afe49", "type": "xiqianvertex",
                 "attributes": "type,acct_stat"}
    model_service = ModelService()
    result_dict = model_service.statistics_operation_attributes_function(dict_data, config_params)
    print(result_dict)


if __name__ == '__main__':
    # main()
    test_function()



import logging

logger_dict = {}


def get_logger(file_name=None):
    if "log" in logger_dict:
        return logger_dict["log"]
    else:
        # 第一步，创建一个logger
        logger = logging.getLogger()
        logger.setLevel(logging.INFO)  # Log等级总开关

        # 第二步，创建一个handler，用于写入日志文件
        if file_name:
            logfile = './log/' + file_name
        else:
            logfile = './ChGraphLog.txt'
        fh = logging.FileHandler(logfile, mode='w')
        fh.setLevel(logging.DEBUG)  # 输出到file的log等级的开关

        # 第三步，再创建一个handler，用于输出到控制台
        ch = logging.StreamHandler()
        ch.setLevel(logging.WARNING)  # 输出到console的log等级的开关

        # 第四步，定义handler的输出格式
        formatter = logging.Formatter("%(asctime)s - %(filename)s[line:%(lineno)d] - %(levelname)s: %(message)s")
        fh.setFormatter(formatter)
        ch.setFormatter(formatter)

        # 第五步，将logger添加到handler里面
        logger.addHandler(fh)
        logger.addHandler(ch)
        logger_dict["log"] = logger
    return logger



#!/usr/bin/env python3
"""
Test psycopg with CockroachDB.
"""

import time
import json
import logging
import psycopg2



# 查询所有图列表
# 查询所有原图列表
# 根据图名查询子图列表
# 根据图名查询schema
def show_graphs(conn, is_source, source_graph_name, graph_name):
    res = []
    con_is_source = ""
    if graph_name is not None and graph_name != '':
        sql = "SELECT graph_create_date,graph_name,graph_cfg,is_source,graph_source FROM name2cfg where graph_status=1 and graph_name=\'"+graph_name+"\'"
    else:
        if is_source is not None and is_source != '':
            con_is_source = " and is_source=\'" + is_source + "\'"

        con_source_graph_name = ""
        if source_graph_name is not None and source_graph_name != '':
            con_source_graph_name = " and graph_source=\'" + source_graph_name + "\'"

        sql = "SELECT graph_create_date,graph_name,graph_cfg,is_source,graph_source FROM name2cfg where graph_status=1 " \
              + con_is_source + con_source_graph_name

    with conn.cursor() as cur:
        cur.execute(sql)
        # logging.debug("print_balances(): status message: %s", cur.statusmessage)
        rows = cur.fetchall()
        conn.commit()

        # print(f"Balances at {time.asctime()}:")
        for row in rows:
            res.append({'graph_create_date': str(row[0]), 'graph_name': row[1],
                        'graph_cfg': row[2], 'is_source': row[3], 'graph_source':row[4]})
        return res

# 图元信息插入
def insert_graph(conn, graph_name, graph_cfg,is_source, source_graph_name):
    sql = "SELECT graph_create_date,graph_name,graph_cfg,is_source,graph_source FROM name2cfg where graph_name=\'"\
          +graph_name+"\' and graph_status=1 "

    with conn.cursor() as cur:
        cur.execute(sql)
        # logging.debug("print_balances(): status message: %s", cur.statusmessage)
        rows = cur.fetchall()
        conn.commit()
        if len(rows) > 0:
            logger.warning("graph name ["+graph_name+"] already exists")
            return "graph name ["+graph_name+"] already exists"
        if is_source == "sub":
            cur.execute("SELECT graph_create_date,graph_name,graph_cfg,is_source,graph_source FROM name2cfg where"+
                        " graph_status=1 and is_source= 'main' and graph_source=\'"+source_graph_name+"\'")
            rows0 = cur.fetchall()
            conn.commit()

            import copy
            subgraph_cfg = json.loads(copy.deepcopy(rows0[0][2]))
            for name in subgraph_cfg["edges"]:
                subgraph_cfg["edges"][name]["db"] = graph_name
            for name in subgraph_cfg["vertexes"]:
                subgraph_cfg["vertexes"][name]["db"] = graph_name
            graph_cfg = json.dumps(subgraph_cfg)

        cur.execute("UPSERT INTO name2cfg values (\'" + time.strftime("%Y-%m-%d %H:%M:%S", time.localtime()) + "\',\'" + graph_name + "\',1,\'" + graph_cfg + "\',\'"+is_source+"\',\'"+source_graph_name+"\');")
        conn.commit()

        logger.info("register graph ["+graph_name+"] done")
        return "register graph ["+graph_name+"] done"

#图元信息逻辑删除
def delete_graph(conn, graph_name):
    sql = "SELECT graph_create_date,graph_name,graph_cfg,is_source,graph_source FROM name2cfg where graph_name=\'"\
          +graph_name+"\' and graph_status=1 "

    with conn.cursor() as cur:
        cur.execute(sql)
        rows = cur.fetchall()
        conn.commit()
        if len(rows) == 0:
            logger.warning("graph name ["+graph_name+"] does not exist")
            return "graph name ["+graph_name+"] does not exist"

        cur.execute("update name2cfg  set graph_status=0 WHERE graph_name=\'"+graph_name+"\' and graph_status=1")
        logger.info("delete graph [" + graph_name + "] done")
        return "delete graph [" + graph_name + "] done"

#图元信息逻辑删除
def update_graph(conn, graph_name, graph_cfg):
    sql = "SELECT graph_create_date,graph_name,graph_cfg,is_source,graph_source FROM name2cfg where graph_name=\'"\
          +graph_name+"\' and graph_status=1 "

    with conn.cursor() as cur:
        cur.execute(sql)
        rows = cur.fetchall()
        conn.commit()
        if len(rows) == 0:
            logger.warning("graph name ["+graph_name+"] does not exist")
            return "graph name ["+graph_name+"] does not exist"

        cur.execute("update name2cfg  set graph_cfg=\'"+graph_cfg+"\' WHERE graph_name=\'"+graph_name+"\' and graph_status=1")


        logger.info("delete graph [" + graph_name + "] done")

    return "delete graph [" + graph_name + "] done"


logger = logging.getLogger('DBoperator')

class DBoperator(object):
    def __init__(self,url):
        self.url = url
        logger.info('DBoperator Start')

        # with conn.cursor() as cur:
        #     cur.execute('create database if not exists data_platform_dev')
        #     cur.execute(
        #         'CREATE TABLE if not exists data_platform_dev.name2cfg (graph_create_date TIMESTAMP NULL,graph_name VARCHAR NULL,graph_status INT2 NULL,	graph_cfg VARCHAR NULL,	is_source VARCHAR NULL,graph_source VARCHAR NULL,	FAMILY "primary" (graph_create_date, graph_name, graph_status, graph_cfg, is_source, rowid, graph_source));')
        #
        #     logging.debug("CREATE TABLE if not exists data_platform_dev.name2cfg: status message: %s", cur.statusmessage)
        # conn.commit()

    def show_tables(self,is_source,source_graph_name,graph_name):
        url = self.url
        conn = psycopg2.connect(url)

        res = show_graphs(conn, is_source, source_graph_name,graph_name)
        print(res)
        conn.close()
        return res

    def insert_tables(self, graph_name, graph_cfg, is_source, source_graph_name):
        url = self.url
        conn = psycopg2.connect(url)

        res = insert_graph(conn, graph_name, graph_cfg, is_source, source_graph_name)
        print(res)
        conn.close()
        return res

    def delete_tables(self,graph_name):
        url = self.url
        conn = psycopg2.connect(url)

        res = delete_graph(conn, graph_name)
        print(res)
        conn.close()
        return res

    def update_tables(self, graph_name, graph_cfg):
        url = self.url
        conn = psycopg2.connect(url)

        res = update_graph(conn, graph_name, graph_cfg)
        print(res)
        conn.close()
        return res

# 图名称切换
    def use_tables(self, graph_name):
        # conn = self.conn
        url = self.url
        conn = psycopg2.connect(url)
        sql = "select graph_cfg from name2cfg where graph_name=\'" + graph_name + "\' and graph_status=1"

        with conn.cursor() as cur:
            cur.execute(sql)
            rows = cur.fetchall()
            conn.commit()
            if len(rows) == 0:
                logger.warning("graph name [" + graph_name + "] does not exist")
                return None
            self.graph_name = graph_name
            self.graph_cfg = json.loads(rows[0][0])
            logger.info("use graph [" + graph_name + "] done")
            conn.close()
        return self.graph_cfg

    def get_name2config(self, graph_name):
        # conn = self.conn
        url = self.url
        conn = psycopg2.connect(url)
        sql = "select * from name2cfg where graph_name=\'" + graph_name + "\' and graph_status=1"

        with conn.cursor() as cur:
            cur.execute(sql)
            rows = cur.fetchall()
            conn.commit()
            if len(rows) == 0:
                logger.warning("graph name [" + graph_name + "] does not exist")
                return None
            self.graph_name = graph_name
            self.name2config = rows
            logger.info("use graph [" + graph_name + "] done")
            conn.close()
        return self.name2config


def main():
    conn = psycopg2.connect('postgresql://root@10.146.143.59:26257/data_platform_dev?sslmode=disable')
    is_source = ''
    source_graph_name = 'a'
    res = show_graphs(conn, is_source, source_graph_name)
    print(res)
    conn.close()

if __name__ == "__main__":
    main()



import os
import sys
import ujson
from clickhouse_driver import Client
from graph_op import CHGraph
from db_op import DBoperator
import psycopg2

'''

graph_dir = "./config/tcpflow_flow.cfg.json"

#graph = CHGraph(graph_dir, client)

'''


def load_config():
    print("服务所有配置开始加载")
    curPath = os.path.abspath(os.path.dirname(__file__))
    rootPath = os.path.split(curPath)
    sys.path.append(rootPath[0])
    sys.path.append(rootPath[0] + "/" + rootPath[1])

    clickhouse_config_dir = rootPath[0] + "/" + rootPath[1] + "/config/" + "graph_config.json"

    with open(clickhouse_config_dir, 'r') as f:
        clickhouse_config = ujson.load(f)

    clickhouse_ip = clickhouse_config["ip"]
    clickhouse_port = clickhouse_config["port"]
    clickhouse_user = clickhouse_config["user"]
    clickhouse_pwd = clickhouse_config["password"]

    db_ip = clickhouse_config["dbip"]
    db_port = clickhouse_config["dbport"]
    db_user = clickhouse_config["dbuser"]
    db_table = clickhouse_config["dbtable"]


    # host = '10.202.255.93', port = '9090', user = 'default', password = 'root'
    print(clickhouse_ip, clickhouse_port, clickhouse_user, clickhouse_pwd)
    # graphClient = Client(host=clickhouse_ip, port=clickhouse_port, user=clickhouse_user, password=clickhouse_pwd)
    clickhouse_connect = {}
    clickhouse_connect["ip"] = clickhouse_ip

    graphClient = Client(host=clickhouse_ip)
    # res = graphClient.execute('show databases')  # 显示所有的数据库
    # print("show databases:", res)
    graph = CHGraph(graphClient)
    url = 'postgresql://'+db_user+'@'+db_ip+':'+db_port+'/'+db_table+'?sslmode=disable'
    # conn = psycopg2.connect(url)
    db = DBoperator(url)
    print("服务所有配置加载结束")
    config_params = {
        "graph": graph,
        "db": db,
        "graphClient": graphClient,
        "clickhouse_connect": clickhouse_connect
    }
    # config_params = {
    #
    # }
    return config_params



import logging

# LOG_FORMAT = "%(asctime)s:%(levelname)s:%(name)s:%(funcName)s:%(message)s"
# logging.basicConfig(filename='./CHGraph.log', level=logging.INFO, format=LOG_FORMAT)
# logger = logging.getLogger('CHGraph')
import Longger_design
logger = Longger_design.get_logger()

class CHGraph(object):

    def __init__(self, client):
        self.client = client
        logger.info('CHGraph Start')

    def execute(self, sql):
        res = self.client.query_dataframe(sql)
        return res



import datetime
from graph_op import CHGraph
from clickhouse_driver import Client
from db_op import DBoperator
import json
import model_sevice as mds
import data_load_config
import numpy as np
import traceback
import logging
import Longger_design

# LOG_FORMAT = "%(asctime)s:%(levelname)s:%(name)s:%(funcName)s:%(message)s"
# logging.basicConfig(filename='./CHGraph.log', level=logging.INFO, format=LOG_FORMAT)
# logger = logging.getLogger()
logger = Longger_design.get_logger()

def get_client(clickhouse_connect):
    graph_client = Client(host=clickhouse_connect["ip"])
    return graph_client


def is_json(json_string):
    try:
        json_object = json.loads(json_string)
    except ValueError:
        return False
    return True


def model_service_ga_build(data, config_params):
    if "graph" in data.keys():
        graphName = data["graph"]
    else:
        graphName = None

    if "sql" in data.keys():
        sql = data["sql"]
    else:
        sql = None

    if "type" in data.keys():
        type = data["type"]
    else:
        type = None

    if "attrType" in data.keys():
        attrType = data["attrType"]
    else:
        attrType = None

    if "subGraph" in data.keys():
        subGraph = data["subGraph"]
    else:
        subGraph = None
    db = config_params["db"]
    clickhouse_connect = config_params["clickhouse_connect"]
    graph_client = get_client(clickhouse_connect)
    graph = CHGraph(graph_client)
    # graph = config_params["graph"]
    # db = config_params["db"]
    graph.use_graph(graphName, db)

    graph_cfg = db.use_tables(graphName)
    #开始时间
    start = datetime.datetime.now()
    db.insert_tables(subGraph, json.dumps(graph_cfg), "sub", graphName)
    #结束时间
    end = datetime.datetime.now()
    logger.info("insert_tables count_times:" + str((end - start).seconds*1000))
    graph.create_subgraph(subGraph)
    dict = graph_cfg[type][attrType]
    graph.use_graph(subGraph, db)
    # 开始时间
    start = datetime.datetime.now()
    graph.update_subgraph_by_sql(subGraph, dict, sql, type, attrType, graphName)
    # 结束时间
    end = datetime.datetime.now()
    logger.info("update_subgraph_by_sql count_times:" + str((end - start).seconds*1000))
    graphNodes = []
    # 开始时间
    start = datetime.datetime.now()
    for vertex in graph_cfg["vertexes"]:
        graphNode = {}
        dict1 = graph_cfg["vertexes"][vertex]
        tem = []
        tem.append(dict1["id"])
        tem.append(dict1["label"])
        for field in dict1["fields"]:
            tem.append(field)
        try:
            vertex_query = graph.query_vertexes(
                vertex,
                [""],
                tem,
                "df"
            )
        except Exception as e:
            print(e)
            logger.error(e)
            return "vertex query failed"
        graphNode["type"] = vertex
        graphNode["data"] = vertex_query

        graphNodes.append(graphNode)
    # 结束时间
    end = datetime.datetime.now()
    logger.info("graphNodes.append count_times:" + str((end - start).seconds*1000))
    graphEdges = []
    # 开始时间
    start = datetime.datetime.now()
    for edge in graph_cfg["edges"]:
        graphEdge = {}
        dict2 = graph_cfg["edges"][edge]
        tem = []
        tem.append(dict2["src"])
        tem.append(dict2["dst"])
        tem.append(dict2["rank"])
        for field in dict2["fields"]:
            tem.append(field)
        try:
            edges_result = graph.query_edges(
                edge,
                [""],
                tem,
                "df"
            )
        except Exception as e:
            print(e)
            logger.error(e)
            return "edge query failed"
        graphEdge["type"] = edge
        graphEdge["data"] = edges_result
        graphEdge["id"] = edge

        graphEdges.append(graphEdge)
    # 结束时间
    end = datetime.datetime.now()
    logger.info("graphEdges.append count_times:" + str((end - start).seconds*1000))
    GraphSet = {}
    GraphSet["pathList"] = {}
    GraphSet["columns"] = []
    GraphSet["rowList"] = []
    GraphSet["pathList"]["graphEdges"] = graphEdges
    GraphSet["pathList"]["graphNodes"] = graphNodes
    return GraphSet


def model_service_graph(data, config_params):
    if "is_source" in data.keys():
        is_source = data["is_source"]
    else:
        is_source = None

    if "source_graph_name" in data.keys():
        source_graph_name = data["source_graph_name"]
    else:
        source_graph_name = None

    if "graph_name" in data.keys():
        graph_name = data["graph_name"]
    else:
        graph_name = None

    db = config_params["db"]
    res = db.show_tables(is_source, source_graph_name, graph_name)
    return res


def model_service_graph_insert(data, config_params):
    if "graph_name" in data.keys():
        graph_name = data["graph_name"]
    else:
        return "graph_name shouldn't be null"

    if "graph_cfg" in data.keys():
        graph_cfg = data["graph_cfg"]
        if not is_json(graph_cfg):
            return "schema is not json string"
    if "is_source" in data.keys():
        is_source = data["is_source"]
    else:
        return "is_source shouldn't be null"

    if "source_graph_name" in data.keys():
        source_graph_name = data["source_graph_name"]
    else:
        return "source_graph_name shouldn't be null"

    db = config_params["db"]
    res = db.insert_tables(graph_name, graph_cfg, is_source, source_graph_name)

    graph = config_params["graph"]
    graph.use_graph(source_graph_name, db)
    graph.create_subgraph(graph_name)

    return res


def model_service_graph_delete(data, config_params):
    if "graph_name" in data.keys():
        graph_name = data["graph_name"]
    else:
        graph_name = "cyber"

    db = config_params["db"]
    res = db.delete_tables(graph_name)

    return res


def model_service_graph_update(data, config_params):
    if "graph_name" in data.keys():
        graph_name = data["graph_name"]
    else:
        graph_name = "cyber"

    if "graph_cfg" in data.keys():
        graph_cfg = data["graph_cfg"]
    else:
        graph_cfg = "cyber"

    db = config_params["db"]
    res = db.update_tables(graph_name, graph_cfg)

    return res


def model_service_graph_search_multi_hop_multi_edge(data, config_params):
    '''

    :param data:
    {
    "start_vertex_list":[
        "10.73.28.115",
        "10.78.55.20"
        ],
    "edge_name_list":[
        "tcpflow",
        "flow"
        ],
    "edge_con_list_list":[["downlink_length>10000", "protocol='http'"], ["record_date='2019-04-15'"]],
    "target_field_list":[
    "record_time"
    ]
}
    :param config_params:
    :return:
    '''

    if "graph_name" in data.keys():
        graph_name = data["graph_name"]
    else:
        graph_name = "cyber"

    # client = Client('10.217.62.41')
    # graph = CHGraph(client)
    graph = config_params["graph"]
    db = config_params["db"]
    graph.use_graph(graph_name, db)

    if "start_vertex_list" in data.keys():
        start_vertex_list = data["start_vertex_list"]
    else:
        start_vertex_list = ['10.73.28.115', '10.78.55.20']

    if "step" in data.keys():
        step = data["step"]
    else:
        step = 1

    if "edge_name_list" in data.keys():
        edge_name_list = data["edge_name_list"]
    else:
        edge_name_list = ["tcpflow", "flow"]

    if "direction" in data.keys():
        direction = data["direction"]
    else:
        direction = "forward"

    if "edge_con_list_list" in data.keys():
        edge_con_list_list = data["edge_con_list_list"]
    else:
        edge_con_list_list = [["downlink_length>10000", "protocol='http'"], ["record_date='2019-04-15'"]]

    if "target_field_list" in data.keys():
        target_field_list = data["target_field_list"]
    else:
        target_field_list = ["record_time"]

    if "only_last_step" in data.keys():
        only_last_step = data["only_last_step"]
    else:
        only_last_step = False

    if "plus_last_vertexes" in data.keys():
        plus_last_vertexes = data["plus_last_vertexes"]
    else:
        plus_last_vertexes = False

    global res
    try:
        res = graph.multi_hop_multi_edge(step,
                                         start_vertex_list,
                                         direction,
                                         edge_name_list,
                                         edge_con_list_list,
                                         target_field_list,
                                         only_last_step,
                                         plus_last_vertexes=plus_last_vertexes
                                         )
    except Exception as e:
        print(e)

    # print(res[1])
    result = dict()
    # result["result"] = res_list2
    # res_list2 = [x.to_dict() for x in res_list2]
    if only_last_step:
        if plus_last_vertexes:
            edge = res[0]
            vertex = res[1]
            for i in range(len(edge_name_list)):
                result["hop_" + str(step)] = {}
                result["hop_" + str(step)][edge_name_list[i]] = edge[i].values.tolist()
            if type(vertex) is np.ndarray:
                result["last_step_vertex"] = vertex.tolist()
            else:
                result["last_step_vertex"] = vertex
        else:
            edge = res
            for i in range(len(edge_name_list)):
                result["hop_" + str(step)] = {}
                result["hop_" + str(step)][edge_name_list[i]] = edge[i].values.tolist()
    else:
        is_empty = True
        is_blank_list = True
        for i in range(len(res)):
            for j in range(len(res[i])):
                is_empty = is_empty & res[i][j].empty
        # if len(res) == 0 or is_empty or res == [[[] for i in range(step)]]:
        if len(res) == 0 or is_empty:
            for i in range(step):
                result["hop_" + str(i + 1)] = {}
                for j in range(len(edge_name_list)):
                    result["hop_" + str(i + 1)][edge_name_list[j]] = []
        else:
            for i in range(step):
                result["hop_" + str(i + 1)] = {}
                for j in range(len(edge_name_list)):
                    if len(res[i]) == 0:
                        result["hop_" + str(i + 1)][edge_name_list[j]] = []
                    else:
                        result["hop_" + str(i + 1)][edge_name_list[j]] = res[i][j].values.tolist()
    return result


def model_service_graph_search_one_hop_multi_edge(data, config_params):
    '''
    {
        "start_vertex_list":[
            "10.73.28.115",
            "10.78.55.20"
            ],
        "edge_name_list":[
            "tcpflow",
            "flow"
            ],
        "edge_con_list_tcpflow":[["downlink_length>10000", "protocol='http'"], ["record_date='2019-04-15'"]],
        "edge_con_list_flow":["record_time"]
    }
    '''

    # print("model_service_graph_search_one_hop_multi_edge")

    if "graph_name" in data.keys():
        graph_name = data["graph_name"]
    else:
        graph_name = "cyber"

    # client = Client('10.217.62.41')
    # graph = CHGraph(client)

    graph = config_params["graph"]

    # graph_dir = "./config/tcpflow_flow.cfg.json"

    # graph = CHGraph(graph_dir, client)

    db = config_params["db"]
    graph.use_graph(graph_name, db)

    if "start_vertex_list" in data.keys():
        start_vertex_list = data["start_vertex_list"]
    else:
        start_vertex_list = ['10.73.28.115', '10.78.55.20']

    if "direction" in data.keys():
        direction = data["direction"]
    else:
        direction = "forward"

    if "edge_name_list" in data.keys():
        edge_name_list = data["edge_name_list"]
    else:
        edge_name_list = ["tcpflow", "flow"]

    if "edge_con_list_list" in data.keys():
        edge_con_list_list = data["edge_con_list_list"]
    else:
        edge_con_list_list = [["downlink_length>10000", "protocol='http'"], ["record_date='2019-04-15'"]]

    if "target_field_list" in data.keys():
        target_field_list = data["target_field_list"]
    else:
        target_field_list = ["record_time"]

    if "end_vertex_con_list" in data.keys():
        end_vertex_con_list = data["end_vertex_con_list"]
    else:
        end_vertex_con_list = None

    # if "edge_con_list_tcpflow" in data.keys():
    #     edge_con_list_tcpflow = data["edge_con_list_tcpflow"]
    # else:
    #     edge_con_list_tcpflow = ["downlink_length>10000", "protocol='http'"]
    #
    # if "edge_con_list_flow" in data.keys():
    #     edge_con_list_flow = data["edge_con_list_flow"]
    # else:
    #     edge_con_list_flow = ["record_date='2019-04-15'"]

    res_list2 = graph.one_hop_multi_edge(start_vertex_list,
                                         direction,
                                         edge_name_list,
                                         edge_con_list_list,
                                         target_field_list,
                                         end_vertex_con_list
                                         )

    print("res_list2")
    print(res_list2)

    result = dict()
    # result["result"] = res_list2
    for i in range(len(edge_name_list)):
        result[edge_name_list[i]] = res_list2[i].values.tolist()

    return result


def model_service_graph_search_one_hop(data, config_params):
    '''
 {
     "start_vertex_list":[
         "10.73.28.115",
         "10.78.55.20"
         ],
     "edge_name":
         "tcpflow"
 }
 '''

    if "graph_name" in data.keys():
        graph_name = data["graph_name"]
    else:
        graph_name = "cyber"

    # client = Client('10.217.62.41')
    # graph = CHGraph(client)
    graph = config_params["graph"]

    # graph_dir = "./config/tcpflow_flow.cfg.json"

    # graph = CHGraph(graph_dir, client)

    db = config_params["db"]
    graph.use_graph(graph_name, db)

    if "start_vertex_list" in data.keys():
        start_vertex_list = data["start_vertex_list"]
    else:
        start_vertex_list = ['10.73.28.115', '10.78.55.20']

    if "direction" in data.keys():
        direction = data["direction"]
    else:
        direction = "forward"

    if "edge_name" in data.keys():
        edge_name = data["edge_name"]
    else:
        edge_name = ["tcpflow", "flow"]

    if "target_field_list" in data.keys():
        target_field_list = data["target_field_list"]
    else:
        target_field_list = ["record_time"]

    if "edge_con_list" in data.keys():
        edge_con_list = data["edge_con_list"]
    else:
        edge_con_list = ["downlink_length>10000", "protocol='http'"]

    if "end_vertex_con_list" in data.keys():
        end_vertex_con_list = data["end_vertex_con_list"]
    else:
        end_vertex_con_list = None

    res_list2 = graph.one_hop(start_vertex_list, direction,
                              edge_name, edge_con_list, target_field_list,
                              end_vertex_con_list)

    # result = dict()
    # result["result"] = res_list2

    return res_list2


def model_service_graph_search_multi_hop(data, config_params):
    '''

    :param data:
      {
    "start_vertex_list":[
        "10.73.28.115",
        "10.78.55.20"
        ],
        "step":2,
    "edge_name":
        "tcpflow",
    "edge_con_list":["downlink_length>10000", "protocol='http'"],
    "target_field_list":[
    "record_time"
    ]
}
    :param config_params:
    :return:
    '''

    if "graph_name" in data.keys():
        graph_name = data["graph_name"]
    else:
        graph_name = "cyber"

    # client = Client('10.217.62.41')
    # graph = CHGraph(client)

    graph = config_params["graph"]

    # graph_dir = "./config/tcpflow_flow.cfg.json"

    # graph = CHGraph(graph_dir, client)
    db = config_params["db"]
    graph.use_graph(graph_name, db)

    if "step" in data.keys():
        step = data["step"]
    else:
        step = 1

    if "start_vertex_list" in data.keys():
        start_vertex_list = data["start_vertex_list"]
    else:
        start_vertex_list = ['10.73.28.115', '10.78.55.20']

    if "direction" in data.keys():
        direction = data["direction"]
    else:
        direction = "forward"

    if "edge_name" in data.keys():
        edge_name = data["edge_name"]
    else:
        edge_name = "tcpflow"

    if "edge_con_list" in data.keys():
        edge_con_list = data["edge_con_list"]
    else:
        edge_con_list = ["downlink_length>10000", "protocol='http'"]

    if "target_field_list" in data.keys():
        target_field_list = data["target_field_list"]
    else:
        target_field_list = ["record_time"]

    if "plus_last_vertexes" in data.keys():
        plus_last_vertexes = data["plus_last_vertexes"]
    else:
        plus_last_vertexes = False

    if "end_vertex_con_list" in data.keys():
        end_vertex_con_list = data["end_vertex_con_list"]
    else:
        end_vertex_con_list = None

    if "only_last_step" in data.keys():
        only_last_step = data["only_last_step"]
    else:
        only_last_step = True

    res_list2 = graph.multi_hop(
        step,
        start_vertex_list,
        direction,
        edge_name,
        edge_con_list,
        target_field_list,
        only_last_step,
        plus_last_vertexes=plus_last_vertexes,
        end_vertex_con_list=end_vertex_con_list)

    # result = dict()
    # result["result"] = res_list2

    return res_list2


def model_service_graph_search_multi_hop_common_vertexes(data, config_params):
    '''

    :param data:
    {
       "step":1,
       "start_vertex_list":[
           "10.73.28.115",
           "10.78.55.20"
           ],
       "edge_name":
           "tcpflow",
       "edge_con_list":[ "protocol='http'"]
   }
    :param config_params:
    :return:
    '''

    if "graph_name" in data.keys():
        graph_name = data["graph_name"]
    else:
        graph_name = "cyber"

    # client = Client('10.217.62.41')
    # graph = CHGraph(client)

    graph = config_params["graph"]

    db = config_params["db"]
    graph.use_graph(graph_name, db)

    if "step" in data.keys():
        step = data["step"]
    else:
        step = 1

    if "direction" in data.keys():
        direction = data["direction"]
    else:
        direction = "forward"

    if "start_vertex_list" in data.keys():
        start_vertex_list = data["start_vertex_list"]
    else:
        start_vertex_list = ['10.73.28.115', '10.78.55.20']

    if "edge_name" in data.keys():
        edge_name = data["edge_name"]
    else:
        edge_name = "tcpflow"

    if "edge_con_list" in data.keys():
        edge_con_list = data["edge_con_list"]
    else:
        edge_con_list = ["protocol='http'"]

    res_list2 = graph.multi_hop_common_vertexes(step,
                                                start_vertex_list,
                                                direction,
                                                edge_name,
                                                edge_con_list)

    # result = dict()
    # result["result"] = res_list2

    return res_list2


def model_service_graph_search_match_edge(data, config_params):
    '''

    :param data:
     {
    "edge_name":
        "tcpflow",
    "edge_con_list":["downlink_length>100000000"],
    "target_field_list" :["record_time", "downlink_length"]
}
    :param config_params:
    :return:
    '''

    if "graph_name" in data.keys():
        graph_name = data["graph_name"]
    else:
        graph_name = "cyber"

    # client = Client('10.217.62.41')
    # graph = CHGraph(client)
    graph = config_params["graph"]

    # graph_dir = "./config/tcpflow_flow.cfg.json"

    # graph = CHGraph(graph_dir, client)

    db = config_params["db"]
    graph.use_graph(graph_name, db)

    if "edge_name" in data.keys():
        edge_name = data["edge_name"]
    else:
        edge_name = "tcpflow"

    if "edge_con_list" in data.keys():
        edge_con_list = data["edge_con_list"]
    else:
        edge_con_list = None

    if "target_field_list" in data.keys():
        target_field_list = data["target_field_list"]
    else:
        target_field_list = ['record_time', 'downlink_length']

    res_list2 = graph.match_edge(edge_name, edge_con_list, target_field_list)

    # result = dict()
    # result["result"] = res_list2

    return res_list2


def model_service_graph_search_match_vertex(data, config_params):
    if "graph_name" in data.keys():
        graph_name = data["graph_name"]
    else:
        graph_name = "cyber"

    # client = Client('10.217.62.41')
    # graph = CHGraph(client)

    graph = config_params["graph"]

    # graph_dir = "./config/tcpflow_flow.cfg.json"

    # graph = CHGraph(graph_dir, client)

    db = config_params["db"]
    graph.use_graph(graph_name, db)

    if "vertex_name" in data.keys():
        vertex_name = data["vertex_name"]
    else:
        vertex_name = 'tcp'

    if "vertex_con_list" in data.keys():
        vertex_con_list = data["vertex_con_list"]
    else:
        vertex_con_list = ["downlink_length>100000000"]

    if "target_field_list" in data.keys():
        target_field_list = data["target_field_list"]
    else:
        target_field_list = ['record_time']

    res_list2 = graph.match_vertex(vertex_name, vertex_con_list, target_field_list)

    return res_list2


def model_service_graph_search_multi_hop_multi_edge_common_vertexes(data, config_params):
    '''

    :param data:
    {
       "step":2,
       "start_vertex_list":[
           "10.73.28.115",
           "10.78.55.20"
           ],
       "edge_name_list":
           ["tcpflow", "flow"],
       "edge_con_list":[["downlink_length>10000", "protocol='http'"], ["record_date='2019-04-15'"]]
   }
    :param config_params:
    :return:
    '''

    if "graph_name" in data.keys():
        graph_name = data["graph_name"]
    else:
        graph_name = "cyber"

    # client = Client('10.217.62.41')
    # graph = CHGraph(client)
    graph = config_params["graph"]
    db = config_params["db"]
    graph.use_graph(graph_name, db)

    if "start_vertex_list" in data.keys():
        start_vertex_list = data["start_vertex_list"]
    else:
        start_vertex_list = ['10.73.28.115', '10.78.55.20']

    if "step" in data.keys():
        step = data["step"]
    else:
        step = 1

    if "edge_name_list" in data.keys():
        edge_name_list = data["edge_name_list"]
    else:
        edge_name_list = ["tcpflow", "flow"]

    if "direction" in data.keys():
        direction = data["direction"]
    else:
        direction = "forward"

    if "edge_con_list_list" in data.keys():
        edge_con_list_list = data["edge_con_list_list"]
    else:
        edge_con_list_list = [["downlink_length>10000", "protocol='http'"], ["record_date='2019-04-15'"]]

    res_list2 = graph.multi_hop_multi_edge_common_vertexes(step,
                                                           start_vertex_list,
                                                           direction,
                                                           edge_name_list,
                                                           edge_con_list_list,
                                                           )

    return res_list2


def model_service_graph_search_find_path(data, config_params):
    if "graph_name" in data.keys():
        graph_name = data["graph_name"]
    else:
        graph_name = "cyber"

    # client = Client('10.217.62.41')
    # graph = CHGraph(client)
    graph = config_params["graph"]
    db = config_params["db"]
    graph.use_graph(graph_name, db)

    if "start_vertex" in data.keys():
        start_vertex = data["start_vertex"]
    else:
        start_vertex = "10.73.28.115"

    if "end_vertex" in data.keys():
        end_vertex = data["end_vertex"]
    else:
        end_vertex = "10.78.55.20"

    if "edge_name_list" in data.keys():
        edge_name_list = data["edge_name_list"]
    else:
        edge_name_list = ["tcpflow", "flow"]

    if "edge_con_list_list" in data.keys():
        edge_con_list_list = data["edge_con_list_list"]
    else:
        edge_con_list_list = [["downlink_length>10000", "protocol='http'"], ["record_date='2019-04-15'"]]

    if "target_field_list" in data.keys():
        target_field_list = data["target_field_list"]
    else:
        target_field_list = ["record_time", "downlink_length"]

    if "step_limit" in data.keys():
        step_limit = data["step_limit"]
    else:
        step_limit = 2

    res_list2 = graph.find_path_multi_edge(start_vertex, end_vertex, edge_name_list, edge_con_list_list,
                                           target_field_list, step_limit)

    # print("res_list2")
    # print(res_list2)

    # result = dict()

    # result["result"] = res_list2

    return res_list2


def model_service_register_graph(data, config_params):
    if "graph_name" in data.keys():
        graph_name = data["graph_name"]
    else:
        graph_name = "cyber"

    if "graph_cfg" in data.keys():
        graph_cfg = data["graph_cfg"]
        if not is_json(graph_cfg):
            return "schema is not json string"

    graph = config_params["graph"]
    res = graph.register_graph(graph_name, graph_cfg)

    return res


def model_service_delete_graph(data, config_params):
    if "graph_name" in data.keys():
        graph_name = data["graph_name"]
    else:
        graph_name = "cyber"

    # client = Client('10.217.62.41')
    # graph = CHGraph(client)
    graph = config_params["graph"]

    res = graph.delete_graph(graph_name)

    return res


def model_service_show_graph(config_params):
    # client = Client('10.217.62.41')
    #
    # graph = CHGraph(client)
    graph = config_params["graph"]

    res = graph.show_graph()

    return list(res["graph_name"])


'''
def model_service_summary_graph(data, config_params):

    if "graph_name" in data.keys():
        graph_name = data["graph_name"]
    else:
        graph_name = "cyber"

    graph = config_params["graph"]

    res = graph.summary_graph(graph_name)

    return res
'''


def model_service_summary_graph(graph_name, config_params):
    client = Client('10.217.62.41')

    graph = CHGraph(client)

    res = graph.summary_graph(graph_name)

    if res != None:
        return {"edge": res[0], "vertex": res[1]}
    else:
        return "graph name [" + graph_name + "] does not exist"


def model_service_description_graph(graph_name, config_params):
    # client = Client('10.217.62.41')
    # graph = CHGraph(client)
    graph = config_params["graph"]

    res = graph.describe_graph(graph_name)

    return res


def model_service_insert_edge(data, config_params):
    if "graph_name" in data.keys():
        graph_name = data["graph_name"]
    else:
        graph_name = "cyber_plus"

    # client = Client('10.217.62.41')
    # graph = CHGraph(client)
    graph = config_params["graph"]

    if "edge_name" in data.keys():
        edge_name = data["edge_name"]
    else:
        edge_name = "tcpflow"

    if "edge_schema" in data.keys():
        edge_schema = data["edge_schema"]
    else:
        edge_schema = ["record_time", "record_date", "source_ip", "destination_ip", "protocol", "destination_port",
                       "uplink_length", "downlink_length"]

    if "edge_data" in data.keys():
        edge_data = data["edge_data"]
        for i, item_edge in enumerate(edge_data):
            edge_data[i] = tuple(item_edge)
    else:
        edge_data = [("2019-04-11 18:48:59", "2019-04-11", "10.66.18.32", "184.173.90.200", "http", "80", 14725, 3116)]

    db = config_params["db"]
    graph.use_graph(graph_name, db)
    res = graph.insert_edge(edge_name, edge_schema, edge_data)

    return res


def model_service_insert_vertex(data, config_params):
    if "graph_name" in data.keys():
        graph_name = data["graph_name"]
    else:
        graph_name = "cyber_plus"

    # client = Client('10.217.62.41')
    # graph = CHGraph(client)
    graph = config_params["graph"]

    if "vertex_name" in data.keys():
        vertex_name = data["vertex_name"]
    else:
        vertex_name = "ip"

    if "vertex_schema" in data.keys():
        vertex_schema = data["vertex_schema"]
    else:
        vertex_schema = ["service_date", "ip", "host", "speed"]

    if "vertex_data" in data.keys():
        vertex_data = data["vertex_data"]
        for i, item_vertex in enumerate(vertex_data):
            vertex_data[i] = tuple(item_vertex)
    else:
        vertex_data = [("2021-01 04", "1.1.1.1", "p47708v.hulk.shbt.qihoo.net", "2"),
                       ("2021-01-05", "1.1.1.2", "p47709v.hulk.shbt.qihoo.net", "3")]

    db = config_params["db"]
    graph.use_graph(graph_name, db)
    res = graph.insert_vertex(vertex_name, vertex_schema, vertex_data)

    return res


def model_service_path_finding(data, config_params):
    if "graph_name" in data.keys():
        graph_name = data["graph_name"]
    else:
        graph_name = "cyber_plus"

    # client = Client('10.217.62.41')
    #
    # graph = CHGraph(client)
    graph = config_params["graph"]

    if "start_vertex_list" in data.keys():
        start_vertex_list = data["start_vertex_list"]
    else:
        start_vertex_list = []

    if "end_vertex_list" in data.keys():
        end_vertex_list = data["end_vertex_list"]
    else:
        end_vertex_list = ['115.102.0.56']

    if "edge_name_list" in data.keys():
        edge_name_list = data["edge_name_list"]
    else:
        edge_name_list = ["service_date", "ip", "host", "speed"]

    if "edge_con_list_list" in data.keys():
        edge_con_list_list = data["edge_con_list_list"]
    else:
        edge_con_list_list = []

    if "target_field_list" in data.keys():
        target_field_list = data["target_field_list"]
    else:
        target_field_list = ['record_time']

    if "step_limit" in data.keys():
        step_limit = data["step_limit"]
    else:
        step_limit = 1

    db = config_params["db"]
    graph.use_graph(graph_name, db)

    res = graph.find_path_multi_edge(start_vertex_list, end_vertex_list, edge_name_list, edge_con_list_list,
                                     target_field_list, step_limit)

    print("res3")
    print(res)

    # try:
    #    res = graph.find_path_multi_edge(start_vertex_list, end_vertex_list, edge_name_list, edge_con_list_list, target_field_list, step_limit)
    # except Exception as e:
    #    print(e)
    #    return "Find path multi edge failed."

    result = {}

    if step_limit == 1:
        result["hop_" + str(1)] = {}
        for j in range(len(edge_name_list)):
            result["hop_" + str(1)][edge_name_list[j]] = res[0][j].values.tolist()
    elif step_limit >= 2:
        for i in range(1, step_limit + 1):
            result["s_" + str(i)] = {}
            if i == 1:
                result["s_" + str(i)] = {}
                for j in range(len(edge_name_list)):
                    result["s_" + str(i)][edge_name_list[j]] = res[i - 1][j].values.tolist()
            else:
                result["s_" + str(i)] = {}
                for h in range(step_limit):
                    result["s_" + str(i)]["hop_" + str(h + 1)] = {}
                    for j in range(len(edge_name_list)):
                        if h < len(res[i - 1]):
                            result["s_" + str(i)]["hop_" + str(h + 1)][edge_name_list[j]] = res[i - 1][h][
                                j].values.tolist()
                            result["s_" + str(i)]["hop_" + str(h + 1)][edge_name_list[j]] = res[i - 1][h][
                                j].values.tolist()
                        else:
                            result["s_" + str(i)]["hop_" + str(h + 1)][edge_name_list[j]] = []
                            result["s_" + str(i)]["hop_" + str(h + 1)][edge_name_list[j]] = []

    return result


def model_service_create_subgraph(data, config_params):
    if "graph_name" in data.keys():
        graph_name = data["graph_name"]
    else:
        graph_name = "taobao"

    if "subgraph_name" in data.keys():
        subgraph_name = data["subgraph_name"]
    else:
        subgraph_name = "taobao_sub"

    # client = Client('10.217.62.41')
    #
    # graph = CHGraph(client)
    graph = config_params["graph"]

    db = config_params["db"]
    graph.use_graph(graph_name, db)

    try:
        status = graph.create_subgraph(subgraph_name)
        return status
    except Exception:
        return "subgraph create failed"


def model_service_update_subgraph_by_multi_hop_multi_edge(data, config_params):
    if "graph_name" in data.keys():
        graph_name = data["graph_name"]
    else:
        # graph_name = "taobao"
        # return None
        pass

    # client = Client('10.217.62.41')
    #
    # graph = CHGraph(client)
    # graph = config_params["graph"]

    db = config_params["db"]
    clickhouse_connect = config_params["clickhouse_connect"]
    graph_client = get_client(clickhouse_connect)
    graph = CHGraph(graph_client)
    if "subgraph_name" in data.keys():
        subgraph_name = data["subgraph_name"]
    else:
        # subgraph_name = "taobao_sub"
        return None

    if "start_vertex_list" in data.keys():
        start_vertex_list = data["start_vertex_list"]
    else:
        start_vertex_list = ['10.73.28.115', '10.78.55.20']

    if "step" in data.keys():
        step = data["step"]
    else:
        step = 1

    if "edge_name_list" in data.keys():
        edge_name_list = data["edge_name_list"]
    else:
        return None

    if "node_name_list" in data.keys():
        node_name_list = data["node_name_list"]
    else:
        return None

    if "direction" in data.keys():
        direction = data["direction"]
    else:
        direction = "forward"

    if "edge_con_list_list" in data.keys():
        edge_con_list_list = data["edge_con_list_list"]
    else:
        # edge_con_list_list = [["downlink_length>10000", "protocol='http'"], ["record_date='2019-04-15'"]]
        edge_con_list_list = [[]]
    # client = Client('10.217.62.41')
    # graph = CHGraph(client)

    # status = graph.update_subgraph_by_multi_hop_multi_edge(
    #       subgraph_name,
    #       step,
    #       start_vertex_list,
    #       direction,
    #       edge_name_list,
    #       edge_con_list_list
    #   )

    try:
        name2config = db.get_name2config(subgraph_name)
        main_graph = name2config[0][5]
        graph_schema = name2config[0][3]
        schema = json.loads(graph_schema)
        graph.use_graph(main_graph, db)

        res = graph.update_subgraph_by_multi_hop_multi_edge(
            main_graph,
            db,
            subgraph_name,
            step,
            start_vertex_list,
            direction,
            edge_name_list,
            edge_con_list_list
        )
        graphEdges = []
        for h in range(step):
            # result["s_" + str(i)]["hop_" + str(h + 1)] = {}
            for j in range(len(edge_name_list)):
                res_tem = res[h][j].values.tolist()
                if len(graphEdges) == len(edge_name_list):
                    graph_edge = graphEdges[j]
                    graph_edge["data"]["detail"] = graph_edge["data"]["detail"] + res_tem
                else:
                    graph_edge = {"data": {"detail": res_tem}}

                graph_edge["type"] = edge_name_list[j]
                schema_tem = res[h][j].columns.values.tolist()
                graph_edge["data"]["schema"] = schema_tem
                if len(graphEdges) < len(edge_name_list):
                    graphEdges.append(graph_edge)
        field_list = schema["vertexes"][node_name_list[0]]["fields"]
        result = model_get_vertex_by_edge(config_params, main_graph, subgraph_name, graphEdges, node_name_list, field_list)
        return result
    except Exception as e:
        traceback.print_exc()
        print(e)
        logger.error(e)
        return "update subgraph by multi hop multi edge failed"


def model_get_vertex_by_edge(config_params, graph_name, subgraph_name, graphEdges, node_name_list, field_list=None):
    node_tem = []
    import numpy as np
    for edge_one in range(len(graphEdges)):
        if graphEdges[edge_one]["data"]["detail"]:
            node_one = np.array(graphEdges[edge_one]["data"]["detail"])[:, 0:2].flatten().tolist()
            node_tem += node_one
    if node_tem:
        node_data = {"subGraph": graph_name, "nodeTypes": node_name_list}
        conditions = []
        if node_name_list:
            for l in range(len(node_name_list)):
                condition = {"type": node_name_list[l], "id": node_tem}
                conditions.append(condition)
            node_data["nodeConditions"] = conditions
        node_data["resultType"] = "def"
        if field_list:
            node_data["fieldList"] = field_list
        model_service = mds.ModelService()
        result = model_service.search_subgraph_by_condition(node_data, config_params)
        graphNodes = result["pathList"]["graphNodes"]
        new_graphNodes = convert_def_node(graphNodes)

        result["pathList"]["graphNodes"] = new_graphNodes
        result["pathList"]["graphEdges"] = graphEdges
    else:
        result = {"pathList": {"graphEdges": graphEdges, "graphNodes": node_tem}}

    return result


def insert_vertexes_data(subgraph_name, graphNodes):
    pass


def convert_def_node(graphNodes):
    new_graphNodes = []
    for graphNode in graphNodes:
        defdata = graphNode["data"]
        #schema = defdata.iloc[:, :2].columns.values.tolist()
        #field_data = defdata.iloc[:, :2].values.tolist()
        schema = defdata.iloc[:, :].columns.values.tolist()
        field_data = defdata.iloc[:, :].values.tolist()
        new_data = {"schema": schema, "detail": field_data}
        new_graphNode = {"data": new_data, "type": graphNode["type"]}
        new_graphNodes.append(new_graphNode)
    return new_graphNodes


def model_service_update_subgraph_by_match_edge(data, config_params):
    if "graph_name" in data.keys():
        graph_name = data["graph_name"]
    else:
        graph_name = "taobao"

    if "subgraph_name" in data.keys():
        subgraph_name = data["subgraph_name"]
    else:
        subgraph_name = "taobao_sub"

    if "edge_name" in data.keys():
        edge_name = data["edge_name"]
    else:
        edge_name = "user_adgroup"

    if "edge_con_list" in data.keys():
        edge_con_list = data["edge_con_list"]
    else:
        edge_con_list = [
            "record_date='2017-05-12'",
            "record_time>'2017-05-12 23:30:00'",
            "pid='430548_1007'"]

    # client = Client('10.217.62.41')
    #
    # graph = CHGraph(client)
    graph = config_params["graph"]
    db = config_params["db"]
    graph.use_graph(graph_name, db)

    try:
        status = graph.update_subgraph_by_match_edge(subgraph_name, edge_name, edge_con_list)
        return status
    except Exception as e:
        print(e)
        return "update subgraph by match edge failed"


def model_service_update_subgraph_by_find_path_multi_edge(data, config_params):
    if "graph_name" in data.keys():
        graph_name = data["graph_name"]
    else:
        # return None
        pass

    if "subgraph_name" in data.keys():
        subgraph_name = data["subgraph_name"]
    else:
        subgraph_name = graph_name + "_sub"

    # client = Client('10.217.62.41')
    #
    # graph = CHGraph(client)
    # graph = config_params["graph"]
    db = config_params["db"]
    clickhouse_connect = config_params["clickhouse_connect"]
    graph_client = get_client(clickhouse_connect)
    graph = CHGraph(graph_client)

    if "start_vertex_list" in data.keys():
        start_vertex_list = data["start_vertex_list"]
    else:
        return None

    if "end_vertex_list" in data.keys():
        end_vertex_list = data["end_vertex_list"]
    else:
        return None

    if "node_name_list" in data.keys():
        node_name_list = data["node_name_list"]
    else:
        node_name_list = []

    if "edge_name_list" in data.keys():
        edge_name_list = data["edge_name_list"]
    else:
        return None

    if "edge_con_list_list" in data.keys():
        edge_con_list_list = data["edge_con_list_list"]
    else:
        edge_con_list_list = [[]]

    if "step_limit" in data.keys():
        step_limit = data["step_limit"]
    else:
        step_limit = 1

    # result = {}
    try:
        name2config = db.get_name2config(subgraph_name)
        main_graph_name = name2config[0][5]
        graph_schema = name2config[0][3]
        schema = json.loads(graph_schema)
        graph.use_graph(main_graph_name, db)

        res = graph.update_subgraph_by_find_path_multi_edge(db,
                                                            main_graph_name,
                                                            subgraph_name,
                                                            start_vertex_list,
                                                            end_vertex_list,
                                                            edge_name_list,
                                                            edge_con_list_list,
                                                            step_limit)
        graphEdges = []
        res_tem = []
        schema_tem = []
        # result["pathList"] = {}
        if step_limit == 1:
            for j in range(len(edge_name_list)):
                graph_edge = {"data": {}}
                res_tem = res[0][j].values.tolist()
                schema_tem = res[0][j].columns.values.tolist()
                graph_edge["type"] = edge_name_list[j]
                graph_edge["data"]["schema"] = schema_tem
                graph_edge["data"]["detail"] = res_tem
                graphEdges.append(graph_edge)
            # result["pathList"]["graphEdges"] = graphEdges
            # result["hop_" + str(1)][edge_name_list[j]] = res[0][j].values.tolist()
        elif step_limit >= 2:
            for i in range(1, step_limit + 1):
                # result["s_" + str(i)] = {}
                if i == 1:
                    # result["s_" + str(i)] = {}
                    for j in range(len(edge_name_list)):
                        graph_edge = {"data": {}}
                        res_tem = res[i - 1][j].values.tolist()
                        schema_tem = res[i - 1][j].columns.values.tolist()
                        graph_edge["type"] = edge_name_list[j]
                        graph_edge["data"]["schema"] = schema_tem
                        graph_edge["data"]["detail"] = res_tem
                        graphEdges.append(graph_edge)
                        # result["s_" + str(i)][edge_name_list[j]] = res[i - 1][j].values.tolist()
                else:
                    # result["s_" + str(i)] = {}
                    for h in range(step_limit):
                        # result["s_" + str(i)]["hop_" + str(h + 1)] = {}
                        for j in range(len(edge_name_list)):
                            if h < len(res[i - 1]):
                                graph_edge = graphEdges[j]
                                res_tem = res[i - 1][h][j].values.tolist()
                                schema_tem = res[i - 1][h][j].columns.values.tolist()
                                graph_edge["type"] = edge_name_list[j]
                                if schema_tem:
                                    graph_edge["data"]["schema"] = schema_tem
                                graph_edge["data"]["detail"] = graph_edge["data"]["detail"] + res_tem
                                # graphEdges.append(graph_edge)
                            else:
                                continue
                                # result["s_" + str(i)]["hop_" + str(h + 1)][edge_name_list[j]] = res[i - 1][h][
                                #     j].values.tolist()
                                # result["s_" + str(i)]["hop_" + str(h + 1)][edge_name_list[j]] = res[i - 1][h][
                                #     j].values.tolist()
                                # else:
                                #     result["s_" + str(i)]["hop_" + str(h + 1)][edge_name_list[j]] = []
                                #     result["s_" + str(i)]["hop_" + str(h + 1)][edge_name_list[j]] = []
                                # mds.ModelService.search_subgraph_by_condition()
        field_list = schema["vertexes"][node_name_list[0]]["fields"]
        result = model_get_vertex_by_edge(config_params, main_graph_name, subgraph_name, graphEdges, node_name_list, field_list)
    except Exception as e:
        print(e)
        print(e.args)
        traceback.print_exc()
        logger.error(e)
        return "subgraph by find path multi edge failed"
    return result


def model_service_destroy_subgraph(data, config_params):
    if "graph_name" in data.keys():
        graph_name = data["graph_name"]
    else:
        graph_name = "cyber"

    if "subgraph_name" in data.keys():
        subgraph_name = data["subgraph_name"]
    else:
        subgraph_name = "taobao_sub"

    # client = Client('10.217.62.41')
    #
    # graph = CHGraph(client)
    graph = config_params["graph"]
    db = config_params["db"]
    graph.use_graph(graph_name, db)

    try:
        status = graph.destroy_subgraph(subgraph_name)
        return status
    except Exception:
        return "destroy subgraph failed"


def model_service_metric_indegree(data, config_params):
    if "graph_name" in data.keys():
        graph_name = data["graph_name"]
    else:
        graph_name = "taobao"

    if "edge_name" in data.keys():
        edge_name = data["edge_name"]
    else:
        edge_name = "user_adgroup"

    if "if_sort" in data.keys():
        if_sort = data["if_sort"]
    else:
        if_sort = False

    if "topk" in data.keys():
        topk = data["topk"]
    else:
        topk = False

    # client = Client('10.217.62.41')
    #
    # graph = CHGraph(client)
    graph = config_params["graph"]
    db = config_params["db"]
    graph.use_graph(graph_name, db)

    try:
        status = graph.metric_indegree(edge_name, if_sort, topk)
        return status
    except Exception:
        return "metric indegree compute failed"


def model_service_metric_outdegree(data, config_params):
    if "graph_name" in data.keys():
        graph_name = data["graph_name"]
    else:
        graph_name = "taobao"

    if "edge_name" in data.keys():
        edge_name = data["edge_name"]
    else:
        edge_name = "user_adgroup"

    if "if_sort" in data.keys():
        if_sort = data["if_sort"]
    else:
        if_sort = False

    if "topk" in data.keys():
        topk = data["topk"]
    else:
        topk = False

    # client = Client('10.217.62.41')
    #
    # graph = CHGraph(client)
    graph = config_params["graph"]
    db = config_params["db"]
    graph.use_graph(graph_name, db)

    try:
        status = graph.metric_outdegree(edge_name, if_sort, topk)
        return status
    except Exception:
        return "metric outdegree compute failed"


def model_service_metric_degree(data, config_params):
    if "graph_name" in data.keys():
        graph_name = data["graph_name"]
    else:
        graph_name = "taobao"

    if "edge_name" in data.keys():
        edge_name = data["edge_name"]
    else:
        edge_name = "user_adgroup"

    if "if_sort" in data.keys():
        if_sort = data["if_sort"]
    else:
        if_sort = False

    if "topk" in data.keys():
        topk = data["topk"]
    else:
        topk = False

    # client = Client('10.217.62.41')
    #
    # graph = CHGraph(client)
    graph = config_params["graph"]
    db = config_params["db"]
    graph.use_graph(graph_name, db)

    try:
        status = graph.metric_degree(edge_name, if_sort, topk)
        return status
    except Exception:
        return "metric degree compute failed"


def model_service_metric_pagerank(data, config_params):
    if "graph_name" in data.keys():
        graph_name = data["graph_name"]
    else:
        graph_name = "taobao"

    if "edge_name" in data.keys():
        edge_name = data["edge_name"]
    else:
        edge_name = "user_adgroup"

    if "if_sort" in data.keys():
        if_sort = data["if_sort"]
    else:
        if_sort = False

    if "num_iter" in data.keys():
        num_iter = data["num_iter"]
    else:
        num_iter = 10

    if "topk" in data.keys():
        topk = data["topk"]
    else:
        topk = -1

    if "d" in data.keys():
        d = data["d"]
    else:
        d = 0.85

    # client = Client('10.217.62.41')
    #
    # graph = CHGraph(client)
    graph = config_params["graph"]
    db = config_params["db"]
    graph.use_graph(graph_name, db)

    try:
        status = graph.metric_pagerank(edge_name, d, num_iter, if_sort, topk)
        return status
    except Exception as e:
        print(e)
        return "metric pagerank compute failed"


def model_service_vertex_match_property(data, config_params):
    if "graph_name" in data.keys():
        graph_name = data["graph_name"]
    else:
        return "graph_name shouldn't be null"

    if "vertex_id_list" in data.keys():
        vertex_id_list = data["vertex_id_list"]
    else:
        return "vertex_id_list shouldn't be null"

    if "vertex_name" in data.keys():
        vertex_name = data["vertex_name"]
    else:
        return "vertex_name shouldn't be null"

    if "vertex_con_list" in data.keys():
        vertex_con_list = data["vertex_con_list"]
    else:
        vertex_con_list = None

    if "target_field_list" in data.keys():
        target_field_list = data["target_field_list"]
    else:
        target_field_list = None
    if "page" in data.keys():
        page = data["page"]
    else:
        page = None
    if "page_size" in data.keys():
        page_size = data["page_size"]
    else:
        page_size = None

    # client = Client('10.217.62.41')
    #
    # graph = CHGraph(client)
    graph = config_params["graph"]
    db = config_params["db"]
    graph.use_graph(graph_name, db)

    try:
        vertex_property = graph.vertex_match_property(vertex_id_list,
                                                      vertex_name,
                                                      vertex_con_list,
                                                      target_field_list,
                                                      page,
                                                      page_size,
                                                      )
        return vertex_property
    except Exception as e:
        print(e)
        return "vertex match property failed"


def model_service_edge_match_property(data, config_params):
    if "graph_name" in data.keys():
        graph_name = data["graph_name"]
    else:
        return "graph_name shouldn't be null"

    if "start_vertex_list" in data.keys():
        start_vertex_list = data["start_vertex_list"]
    else:
        return "start_vertex_list shouldn't be null"

    if "end_vertex_list" in data.keys():
        end_vertex_list = data["end_vertex_list"]
    else:
        return "end_vertex_list shouldn't be null"

    if "edge_name" in data.keys():
        edge_name = data["edge_name"]
    else:
        return "edge_name shouldn't be null"

    if "edge_con_list" in data.keys():
        edge_con_list = data["edge_con_list"]
    else:
        edge_con_list = None

    if "target_field_list" in data.keys():
        target_field_list = data["target_field_list"]
    else:
        target_field_list = None
    if "page" in data.keys():
        page = data["page"]
    else:
        page = None
    if "page_size" in data.keys():
        page_size = data["page_size"]
    else:
        page_size = None
    # client = Client('10.217.62.41')
    #
    # graph = CHGraph(client)
    graph = config_params["graph"]
    db = config_params["db"]
    graph.use_graph(graph_name, db)

    try:
        edge_property = graph.edge_match_property(
            start_vertex_list,
            end_vertex_list,
            edge_name,
            edge_con_list,
            target_field_list,
            page,
            page_size
        )
        return edge_property
    except Exception as e:
        print(e)
        return "edge match property failed"


def model_service_query_vertexes(data, config_params):
    if "graph_name" in data.keys():
        graph_name = data["graph_name"]
    else:
        graph_name = "anti_money_launder"

    if "vertex_name" in data.keys():
        vertex_name = data["vertex_name"]
    else:
        vertex_name = "account"

    if "vertex_con_list" in data.keys():
        vertex_con_list = data["vertex_con_list"]
    else:
        vertex_con_list = None

    if "target_field_list" in data.keys():
        target_field_list = data["target_field_list"]
    else:
        target_field_list = None

    # client = Client('10.217.62.41')
    #
    # graph = CHGraph(client)
    graph = config_params["graph"]
    db = config_params["db"]
    graph.use_graph(graph_name, db)

    try:
        vertex_query = graph.query_vertexes(
            vertex_name,
            vertex_con_list,
            target_field_list,
        )
        return vertex_query
    except Exception as e:
        print(e)
        return "vertex query failed"


def model_service_query_edges(data, config_params):
    if "graph_name" in data.keys():
        graph_name = data["graph_name"]
    else:
        graph_name = "anti_money_launder"

    if "edge_name" in data.keys():
        edge_name = data["edge_name"]
    else:
        edge_name = "transactions"

    if "edge_con_list" in data.keys():
        edge_con_list = data["edge_con_list"]
    else:
        edge_con_list = None

    if "target_field_list" in data.keys():
        target_field_list = data["target_field_list"]
    else:
        target_field_list = None

    # client = Client('10.217.62.41')
    #
    # graph = CHGraph(client)
    graph = config_params["graph"]
    db = config_params["db"]
    graph.use_graph(graph_name, db)

    try:
        edges_result = graph.query_edges(
            edge_name,
            edge_con_list,
            target_field_list,
        )
        return edges_result
    except Exception as e:
        print(e)
        return "edge query failed"


def model_service_time_static_subgraph(data, config_params):
    if "graph_name" in data.keys():
        graph_name = data["graph_name"]
    else:
        graph_name = "anti_money_launder"

    if "time_field" in data.keys():
        time_field = data["time_field"]
    else:
        time_field = "record_date"

    if "edge_name" in data.keys():
        edge_name = data["edge_name"]
    else:
        edge_name = "transactions"

    if "edge_con_list" in data.keys():
        edge_con_list = data["edge_con_list"]
    else:
        edge_con_list = ["record_date < '2017-05-11'"]

    if "time_dimention" in data.keys():
        time_dimention = data["time_dimention"]
    else:
        time_dimention = "Day"

    # client = Client('10.217.62.41')
    #
    # graph = CHGraph(client)
    graph = config_params["graph"]
    db = config_params["db"]
    graph.use_graph(graph_name, db)

    try:
        time_static_result = graph.count_edge_by_time(
            edge_name,
            edge_con_list,
            time_field,
            time_dimention
        )
        return time_static_result
    except Exception as e:
        print(e)
        return "subgraph time static failed"
